@@ -26,12 +26,8 @@
 /* The following are set by pm_init(), then used by subsequent calls to other
    pm_xxx() functions.
    */
-static char* progname;
-static bool pm_show_version;
-    /* Programs should show version information (per the user's
-       --version option) 
-    */
-static bool showmessages;  
+static const char* pm_progname;
+static bool pm_showmessages;  
     /* Programs should display informational messages (because the user didn't
        specify the --quiet option).
     */
@@ -250,11 +246,61 @@ pm_lcm(const unsigned int x,
 
 
 void
-pm_init(int * const argcP, char * argv[]) {
+pm_init(const char * const progname, unsigned int const flags) {
+/*----------------------------------------------------------------------------
+   Initialize static variables that Netpbm library routines use.
 
+   Any user of Netpbm library routines is expected to call this at the
+   beginning of this program, before any other Netpbm library routines.
+
+   A program may call this via pm_proginit() instead, though.
+-----------------------------------------------------------------------------*/
+    pm_setMessage(FALSE, NULL);
+
+    pm_progname = progname;
+    
+#ifdef O_BINARY
+#ifdef HAVE_SETMODE
+    /* Set the stdin and stdout mode to binary.  This means nothing on Unix,
+       but matters on Windows.
+       
+       Note that stdin and stdout aren't necessarily image files.  In
+       particular, stdout is sometimes text for human consumption,
+       typically printed on the terminal.  Binary mode isn't really
+       appropriate for that case.  We do this setting here without
+       any knowledge of how stdin and stdout are being used because it is
+       easy.  But we do make an exception for the case that we know the
+       file is a terminal, to get a little closer to doing the right
+       thing.  
+    */
+    if (!isatty(0)) setmode(0,O_BINARY);  /* Standard Input */
+    if (!isatty(1)) setmode(1,O_BINARY);  /* Standard Output */
+#endif /* HAVE_SETMODE */
+#endif /* O_BINARY */
+
+}
+
+
+
+void
+pm_proginit(int * const argcP, char * argv[]) {
+/*----------------------------------------------------------------------------
+   Do various initialization things that all programs in the Netpbm package,
+   and programs that emulate such programs, should do.
+
+   This includes processing global options.
+
+   This includes calling pm_init() to initialize the Netpbm libraries.
+-----------------------------------------------------------------------------*/
     int argn, i;
     char *rgbdef;
-
+    const char * progname;
+    bool showmessages;
+    bool show_version;
+      /* We're supposed to just show the version information, then exit the
+         program.
+      */
+    
 #ifndef VMS
     /* Extract program name. */
     progname = strrchr( argv[0], '/');
@@ -280,97 +326,94 @@ pm_init(int * const argcP, char * argv[]) {
     else
         ++progname;
 
+    pm_init(progname, 0);
+
     /* Check for any global args. */
     showmessages = TRUE;
-    pm_show_version = FALSE;;
+    show_version = FALSE;
     for ( argn = 1; argn < *argcP; ++argn ) {
         if ( pm_keymatch( argv[argn], "-quiet", 6 ) ||
              pm_keymatch( argv[argn], "--quiet", 7 ) ) {
             showmessages = FALSE;
         } else if ( pm_keymatch( argv[argn], "-version", 8 ) ||
                     pm_keymatch( argv[argn], "--version", 9 ) ) 
-            pm_show_version = TRUE;
+            show_version = TRUE;
         else
             continue;
         for ( i = argn + 1; i <= *argcP; ++i )
             argv[i - 1] = argv[i];
         --(*argcP);
         }
-/* Set the stdin and stdout mode to binary.  This means nothing on Unix,
-   but matters on Windows.
 
-   Note that stdin and stdout aren't necessarily image files.  In
-   particular, stdout is sometimes text for human consumption,
-   typically printed on the terminal.  Binary mode isn't really
-   appropriate for that case.  We do this setting here without
-   any knowledge of how stdin and stdout are being used because it is
-   easy.  But we do make an exception for the case that we know the
-   file is a terminal, to get a little closer to doing the right
-   thing.  
-*/
-#ifdef O_BINARY
-#ifdef HAVE_SETMODE
-    if (!isatty(0)) setmode(0,O_BINARY);  /* Standard Input */
-    if (!isatty(1)) setmode(1,O_BINARY);  /* Standard Output */
-#endif /* HAVE_SETMODE */
-#endif /* O_BINARY */
+    pm_setMessage((unsigned int) showmessages, NULL);
 
-    if (pm_show_version) {
-            pm_message( "Using libpbm from Netpbm Version: %s", PBMPLUS_VERSION );
+    if (show_version) {
+        pm_message( "Using libpbm from Netpbm Version: %s", NETPBM_VERSION );
 #if defined(COMPILE_TIME) && defined(COMPILED_BY)
-            pm_message( "Compiled %s by user \"%s\"",
-                        COMPILE_TIME, COMPILED_BY );
+        pm_message( "Compiled %s by user \"%s\"",
+                    COMPILE_TIME, COMPILED_BY );
 #endif
 #ifdef BSD
-            pm_message( "BSD defined" );
+        pm_message( "BSD defined" );
 #endif /*BSD*/
 #ifdef SYSV
 #ifdef VMS
-            pm_message( "VMS & SYSV defined" );
+        pm_message( "VMS & SYSV defined" );
 #else
-            pm_message( "SYSV defined" );
+        pm_message( "SYSV defined" );
 #endif
 #endif /*SYSV*/
 #ifdef MSDOS
-            pm_message( "MSDOS defined" );
+        pm_message( "MSDOS defined" );
 #endif /*MSDOS*/
 #ifdef AMIGA
-            pm_message( "AMIGA defined" );
+        pm_message( "AMIGA defined" );
 #endif /* AMIGA */
 #ifdef PBMPLUS_BROKENPUTC1
-            pm_message( "PBMPLUS_BROKENPUTC1 defined" );
+        pm_message( "PBMPLUS_BROKENPUTC1 defined" );
 #endif /*PBMPLUS_BROKENPUTC1*/
 #ifdef PBMPLUS_BROKENPUTC2
-            pm_message( "PBMPLUS_BROKENPUTC2 defined" );
+        pm_message( "PBMPLUS_BROKENPUTC2 defined" );
 #endif /*PBMPLUS_BROKENPUTC2*/
 #ifdef PPM_PACKCOLORS
-            pm_message( "PPM_PACKCOLORS defined" );
+        pm_message( "PPM_PACKCOLORS defined" );
 #endif /*PPM_PACKCOLORS*/
 #ifdef DEBUG
-            pm_message( "DEBUG defined" );
+        pm_message( "DEBUG defined" );
 #endif /*DEBUG*/
 #ifdef NEED_VFPRINTF1
-            pm_message( "NEED_VFPRINTF1 defined" );
+        pm_message( "NEED_VFPRINTF1 defined" );
 #endif /*NEED_VFPRINTF1*/
 #ifdef NEED_VFPRINTF2
-            pm_message( "NEED_VFPRINTF2 defined" );
+        pm_message( "NEED_VFPRINTF2 defined" );
 #endif /*NEED_VFPRINTF2*/
-            pm_message( "RGB_ENV='%s'", RGBENV );
-            rgbdef = getenv(RGBENV);
-            if( rgbdef )
-                pm_message( "RGBENV= '%s' (env vbl set to '%s')", 
-                            RGBENV, rgbdef );
-            else
-                pm_message( "RGBENV= '%s' (env vbl is unset)", RGBENV);
-            exit( 0 );
-    }
+        pm_message( "RGB_ENV='%s'", RGBENV );
+        rgbdef = getenv(RGBENV);
+        if( rgbdef )
+            pm_message( "RGBENV= '%s' (env vbl set to '%s')", 
+                        RGBENV, rgbdef );
+        else
+            pm_message( "RGBENV= '%s' (env vbl is unset)", RGBENV);
+        exit( 0 );
     }
+}
+
+
+void
+pm_setMessage(int const newState, int * const oldStateP) {
+    
+    if (oldStateP)
+        *oldStateP = pm_showmessages;
+
+    pm_showmessages = !!newState;
+}
+
 
 /* Error handling. */
 
 void
 pm_usage( const char usage[] ) {
-    fprintf( stderr, "usage:  %s %s\n", progname, usage );
+    fprintf( stderr, "usage:  %s %s\n", pm_progname, usage );
     exit( 1 );
     }
 
@@ -397,9 +440,9 @@ pm_message(const char format[], ... ) {
 
     va_start( args, format );
 
-    if ( showmessages )
+    if ( pm_showmessages )
         {
-        fprintf( stderr, "%s: ", progname );
+        fprintf( stderr, "%s: ", pm_progname );
         (void) vfprintf( stderr, format, args );
         fputc( '\n', stderr );
         }
@@ -412,7 +455,7 @@ pm_error(const char format[], ... ) {
 
     va_start( args, format );
 
-    fprintf( stderr, "%s: ", progname );
+    fprintf( stderr, "%s: ", pm_progname );
     (void) vfprintf( stderr, format, args );
     fputc( '\n', stderr );
     va_end( args );
@@ -734,7 +777,7 @@ char *pm_read_unknown_size(FILE * const file, long * const nread) {
             else
                 nalloc += nalloc;
             if ((buf=realloc(buf, nalloc)) == NULL)
-                pm_error("Cannot allocate %d bytes of memory", nalloc);
+                pm_error("Cannot allocate %ld bytes of memory", nalloc);
         }
 
         if ((val = getc(file)) == EOF)
