@@ -301,7 +301,7 @@ alloc_coder (char const * const *inputname, const c_options_t *options,
    c->options.images_level = min (c->options.images_level,
 				  c->options.lc_max_level - 1);
    
-   c->products_level  = max (0, (c->options.lc_max_level
+   c->products_level  = max (0, ((signed int) c->options.lc_max_level
 				 - c->options.images_level - 1));
    c->pixels 	      = Calloc (size_of_level (c->options.lc_max_level),
 				sizeof (real_t));
@@ -695,7 +695,6 @@ pattern2type (unsigned frame, const char *pattern)
         break;
     default:
         error ("Frame type %c not valid. Choose one of I,B or P.", tmp);
-        retval = NONE;
     }
     return retval;
 }
