@@ -114,7 +114,7 @@ parseargs (param_t *usr_params, int argc, char **argv, const char *synopsis,
        "Print brief help, then exit."},
       {"help", NULL, 'H', PFLAG, {0}, NULL,
        "Print detailed help, then exit."},
-      {NULL, NULL, 0, 0, {0}, NULL, NULL }
+      {NULL, NULL, 0, PSTR, {0}, NULL, NULL }
    };
    param_t    short_sys_params [] =	/* short system parameters */
    {
@@ -126,7 +126,7 @@ parseargs (param_t *usr_params, int argc, char **argv, const char *synopsis,
        "Load `%s' to initialize parameters."},
       {"help", NULL, 'h', PFLAG, {0}, NULL,
        "Print this help, then exit."},
-      {NULL, NULL, 0, 0, {0}, NULL, NULL }
+      {NULL, NULL, 0, PSTR, {0}, NULL, NULL }
    };
    char *sys_path;			/* path to system config file */
 
@@ -325,8 +325,9 @@ parseargs (param_t *usr_params, int argc, char **argv, const char *synopsis,
 	       exit (2);
 	    }
 	    else if (streq (params [param_index].name, "verbose"))
-	       fiasco_set_verbosity (* (int *) parameter_value (params,
-								"verbose"));
+	       fiasco_set_verbosity (
+               * (fiasco_verbosity_e *) parameter_value (params,
+                                                         "verbose"));
 	    else if (streq (params [param_index].name, "config"))
 	       read_config_file = YES;
 	    param_index = -1;		/* clear index flag */
