@@ -68,6 +68,83 @@ stripeq(const char * const comparand, const char * const comparator) {
 
 
 
+int
+pnm_tupleequal(const struct pam * const pamP, 
+               tuple              const comparand, 
+               tuple              const comparator) {
+
+    /* TODO: For speed reasons, we should probably make this an inline */
+
+    unsigned int plane;
+    bool equal;
+
+    equal = TRUE;  /* initial value */
+    for (plane = 0; plane < pamP->depth; ++plane) 
+        if (comparand[plane] != comparator[plane])
+            equal = FALSE;
+
+    return equal;
+}
+
+
+
+
+void
+pnm_assigntuple(const struct pam * const pamP,
+                tuple              const dest,
+                tuple              const source) {
+
+    /* TODO: For speed reasons, we should probably make this an inline */
+
+    unsigned int plane;
+    for (plane = 0; plane < pamP->depth; ++plane) {
+        dest[plane] = source[plane];
+    }
+}
+
+
+
+void
+pnm_scaletuple(const struct pam * const pamP,
+               tuple              const dest,
+               tuple              const source, 
+               sample             const newmaxval) {
+
+    /* TODO: For speed reasons, we should probably make this an inline */
+
+    unsigned int plane;
+    for (plane = 0; plane < pamP->depth; ++plane) 
+        dest[plane] = 
+            (source[plane] * newmaxval + (pamP->maxval/2)) / pamP->maxval;
+}
+
+
+ 
+void
+createBlackTuple(const struct pam * const pamP, tuple * const blackTupleP) {
+/*----------------------------------------------------------------------------
+   Create a "black" tuple.  By that we mean a tuple all of whose elements
+   are zero.  If it's an RGB, grayscale, or b&w pixel, that means it's black.
+-----------------------------------------------------------------------------*/
+    *blackTupleP = pnm_allocpamtuple(pamP);
+    if (pamP->format == PAM_FORMAT) {
+        /* In this format, we don't know the meaning of "black", so we
+           just punt.
+           */
+        int i;
+        for (i = 0; i < pamP->depth; i++) 
+            (*blackTupleP)[i] = 0;
+    } else {
+        xel black_xel;
+        black_xel = pnm_blackxel(pamP->maxval, pamP->format);
+        (*blackTupleP)[0] = PPM_GETR(black_xel);
+        (*blackTupleP)[1] = PPM_GETG(black_xel);
+        (*blackTupleP)[2] = PPM_GETB(black_xel);
+    }
+}
+
+
+
 tuple *
 pnm_allocpamrow(struct pam * const pamP) {
     const int bytes_per_tuple = pamP->depth * sizeof(sample);
@@ -115,8 +192,8 @@ pnm_allocpamarray(struct pam * const pamP) {
     
     tuplearray = malloc(pamP->height * sizeof(tuple*));
     if (tuplearray == NULL) 
-        pm_error("Out of storage allocating the row pointer section of "
-                 "a %d row array", pamP->height);
+        pm_error("Out of memory allocating the row pointer section of "
+                 "a %u row array", pamP->height);
 
     for (row = 0; row < pamP->height; row++) {
         tuplearray[row] = pnm_allocpamrow(pamP);
@@ -344,7 +421,7 @@ pnm_readpaminit(FILE *file, struct pam * const pamP, const int size) {
 
     case PBM_TYPE:
         pbm_readpbminitrest(pamP->file, &pamP->width,&pamP->height);
-        pamP->maxval = (sample) pnm_pbmmaxval;
+        pamP->maxval = (sample) 1;
         pamP->depth = 1;
         strcpy(pamP->tuple_type, PAM_PBM_TUPLETYPE);
         break;
@@ -384,7 +461,7 @@ get_sample(FILE *file, const int bytes) {
         int n_read;
 
         n_read = fread(inval, bytes, 1, file);
-        if (n_read < bytes) 
+        if (n_read < 1) 
             pm_error("EOF/error reading %d byte sample from file.", bytes);
         value = 0;  /* initial value */
         cursor = 0;
@@ -450,7 +527,17 @@ pnm_readpam(FILE *file, struct pam * const pamP, const int size) {
 
 
 void 
-pnm_writepam(struct pam *const pamP, const tuple * const * const tuplearray);
+pnm_writepam(struct pam *const pamP, tuple ** const tuplearray) {
+
+    int row;
+
+    pnm_writepaminit(pamP);
+    
+    for (row = 0; row < pamP->height; row++) 
+        pnm_writepamrow(pamP, tuplearray[row]);
+}
+
+
 
 void 
 pnm_writepaminit(struct pam *const pamP) {
@@ -491,7 +578,7 @@ pnm_writepaminit(struct pam *const pamP) {
             pm_error("pnm_writepaminit() got PPM format, but depth = %d "
                      "instead of 3, as required for PPM.", pamP->depth);
         if (pamP->maxval > PPM_OVERALLMAXVAL) 
-            pm_error("pnm_writepaminit() got PPM format, but maxval = %d, "
+            pm_error("pnm_writepaminit() got PPM format, but maxval = %ld, "
                      "which exceeds the maximum allowed for PPM: %d", 
                      pamP->maxval, PPM_OVERALLMAXVAL);
         ppm_writeppminit(pamP->file, pamP->width, pamP->height, 
@@ -503,7 +590,7 @@ pnm_writepaminit(struct pam *const pamP) {
             pm_error("pnm_writepaminit() got PGM format, but depth = %d "
                      "instead of 1, as required for PGM.", pamP->depth);
         if (pamP->maxval > PGM_OVERALLMAXVAL)
-            pm_error("pnm_writepaminit() got PGM format, but maxval = %d, "
+            pm_error("pnm_writepaminit() got PGM format, but maxval = %ld, "
                      "which exceeds the maximum allowed for PGM: %d", 
                      pamP->maxval, PGM_OVERALLMAXVAL);
         pgm_writepgminit(pamP->file, pamP->width, pamP->height, 
@@ -515,7 +602,7 @@ pnm_writepaminit(struct pam *const pamP) {
             pm_error("pnm_writepaminit() got PBM format, but depth = %d "
                      "instead of 1, as required for PBM.", pamP->depth);
         if (pamP->maxval != 1) 
-            pm_error("pnm_writepaminit() got PBM format, but maxval = %d "
+            pm_error("pnm_writepaminit() got PBM format, but maxval = %ld "
                      "instead of 1, as required for PBM.", pamP->maxval);
         pbm_writepbminit(pamP->file, pamP->width, pamP->height, 0);
         break;
@@ -635,6 +722,9 @@ pnm_checkpam(struct pam *const pamP, const enum pm_check_type check_type,
 
 
 
+
+double pnm_lumin_factor[3] = {PPM_LUMINR, PPM_LUMING, PPM_LUMINB};
+
 void
 pnm_YCbCrtuple(const tuple tuple, 
                double * const YP, double * const CbP, double * const CrP) {
@@ -646,7 +736,7 @@ pnm_YCbCrtuple(const tuple tuple,
     const int grn = tuple[PAM_GRN_PLANE];
     const int blu = tuple[PAM_BLU_PLANE];
     
-    *YP  = (+ 0.2989 * red + 0.5866 * grn + 0.1145 * blu);
+    *YP  = (+ PPM_LUMINR * red + PPM_LUMING * grn + PPM_LUMINB * blu);
     *CbP = (- 0.1687 * red - 0.3312 * grn + 0.5000 * blu);
     *CrP = (+ 0.5000 * red - 0.4183 * grn - 0.0816 * blu);
 }
