@@ -348,7 +348,7 @@ FILE *tfp;
     }
 
     if (verbose) {
-      pm_message ("reading a %dw x %dh image, %d bit%s %s%s%s%s",
+      pm_message ("reading a %ldw x %ldh image, %d bit%s %s%s%s%s",
 		  info_ptr->width, info_ptr->height,
 		  info_ptr->bit_depth, info_ptr->bit_depth > 1 ? "s" : "",
 		  type_string, alpha_string, gamma_string,
