@@ -35,6 +35,7 @@ struct cmdline_info {
     unsigned int ybox;
     unsigned int pixels;
     unsigned int verbose;
+    unsigned int nomix;
 };
 
 
@@ -45,43 +46,47 @@ parse_command_line(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
+    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+        /* Instructions to optParseOptions3 on how to parse our options.
          */
-    optStruct2 opt;
+    optStruct3 opt;
 
     unsigned int option_def_index;
-    int xysize, xsize, ysize, pixels;
+    unsigned int xysize;
+    int xsize, ysize, pixels;
     int reduce;
     float xscale, yscale, scale_parm;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0,   "xsize",     OPT_UINT,    &xsize,         0);
-    OPTENTRY(0,   "width",     OPT_UINT,    &xsize,         0);
-    OPTENTRY(0,   "ysize",     OPT_UINT,    &ysize,         0);
-    OPTENTRY(0,   "height",    OPT_UINT,    &ysize,         0);
-    OPTENTRY(0,   "xscale",    OPT_FLOAT,   &xscale,        0);
-    OPTENTRY(0,   "yscale",    OPT_FLOAT,   &yscale,        0);
-    OPTENTRY(0,   "pixels",    OPT_UINT,    &pixels,        0);
-    OPTENTRY(0,   "xysize",    OPT_FLAG,    &xysize,        0);
-    OPTENTRY(0,   "verbose",   OPT_FLAG,    &cmdline_p->verbose,        0);
-    OPTENTRY(0,   "reduce",    OPT_UINT,    &reduce,        0);
+    OPTENT3(0, "xsize",     OPT_UINT,    &xsize,               NULL, 0);
+    OPTENT3(0, "width",     OPT_UINT,    &xsize,               NULL, 0);
+    OPTENT3(0, "ysize",     OPT_UINT,    &ysize,               NULL, 0);
+    OPTENT3(0, "height",    OPT_UINT,    &ysize,               NULL, 0);
+    OPTENT3(0, "xscale",    OPT_FLOAT,   &xscale,              NULL, 0);
+    OPTENT3(0, "yscale",    OPT_FLOAT,   &yscale,              NULL, 0);
+    OPTENT3(0, "pixels",    OPT_UINT,    &pixels,              NULL, 0);
+    OPTENT3(0, "reduce",    OPT_UINT,    &reduce,              NULL, 0);
+    OPTENT3(0, "xysize",    OPT_FLAG,    NULL, &xysize,              0);
+    OPTENT3(0, "verbose",   OPT_FLAG,    NULL, &cmdline_p->verbose,  0);
+    OPTENT3(0, "nomix",     OPT_FLAG,    NULL, &cmdline_p->nomix,    0);
 
     /* Set the defaults. -1 = unspecified */
+    /* (Now that we're using ParseOptions3, we don't have to do this -1
+       nonsense, but we don't want to risk screwing these complex 
+       option compatibilities up, so we'll convert that later.
+    */
     xsize = -1;
     ysize = -1;
     xscale = -1.0;
     yscale = -1.0;
     pixels = -1;
-    xysize = 0;
     reduce = -1;
-    cmdline_p->verbose = FALSE;
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    pm_optParseOptions2(&argc, argv, opt, 0);
+    pm_optParseOptions3( &argc, argv, opt, sizeof(opt), 0 );
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
     if (xsize == 0)
@@ -363,30 +368,315 @@ horizontal_scale(const xel inputxelrow[], xel newxelrow[],
 }
 
 
+
+static void
+zeroAccum(int const cols, int const format, 
+          float rs[], float gs[], float bs[]) {
+
+    int col;
+
+    for ( col = 0; col < cols; ++col )
+        rs[col] = gs[col] = bs[col] = 0.0;
+}
+
+
+
+static void
+accumOutputRow(xel * const xelrow, float const fraction, 
+               float rs[], float gs[], float bs[], 
+               int const cols, int const format) {
+/*----------------------------------------------------------------------------
+   Take 'fraction' times the color in row xelrow and add it to 
+   rs/gs/bs.  'fraction' is less than 1.0.
+-----------------------------------------------------------------------------*/
+    int col;
+
+    switch ( PNM_FORMAT_TYPE(format) ) {
+    case PPM_TYPE:
+        for ( col = 0; col < cols; ++col ) {
+            rs[col] += fraction * PPM_GETR(xelrow[col]);
+            gs[col] += fraction * PPM_GETG(xelrow[col]);
+            bs[col] += fraction * PPM_GETB(xelrow[col]);
+        }
+        break;
+
+    default:
+        for ( col = 0; col < cols; ++col)
+            gs[col] += fraction * PNM_GET1(xelrow[col]);
+        break;
+    }
+}
+
+
+
+static void
+makeRow(xel * const xelrow, float rs[], float gs[], float bs[],
+        int const cols, xelval const maxval, int const format) {
+/*----------------------------------------------------------------------------
+   Make an xel row at 'xelrow' with format 'format' and
+   maxval 'maxval' out of the color values in 
+   rs[], gs[], and bs[].
+-----------------------------------------------------------------------------*/
+    int col;
+
+    switch ( PNM_FORMAT_TYPE(format) ) {
+    case PPM_TYPE:
+        for ( col = 0; col < cols; ++col) {
+            PPM_ASSIGN(xelrow[col], 
+                       min(maxval, (int) (rs[col] + 0.5)), 
+                       min(maxval, (int) (gs[col] + 0.5)), 
+                       min(maxval, (int) (bs[col] + 0.5))
+                );
+        }
+        break;
+
+    default:
+        for ( col = 0; col < cols; ++col ) {
+            PNM_ASSIGN1(xelrow[col], 
+                        min(maxval, (int) (gs[col] + 0.5)));
+        }
+        break;
+    }
+}
+
+
+
+static void
+scaleWithMixing(FILE * const ifP,
+                int const cols, int const rows,
+                xelval const maxval, int const format,
+                int const newcols, int const newrows,
+                xelval const newmaxval, int const newformat,
+                float const xscale, float const yscale,
+                bool const verbose) {
+/*----------------------------------------------------------------------------
+   Scale the image on input file 'ifP' (which is described by 
+   'cols', 'rows', 'format', and 'maxval') by xscale horizontally and
+   yscale vertically and write the result to standard output as format
+   'newformat' and with maxval 'newmaxval'.
+
+   The input file is positioned past the header, to the beginning of the
+   raster.  The output file is too.
+
+   Mix colors from input rows together in the output rows.
+-----------------------------------------------------------------------------*/
+    /* Here's how we think of the color mixing scaling operation:  
+       
+       First, I'll describe scaling in one dimension.  Assume we have
+       a one row image.  A raster row is ordinarily a sequence of
+       discrete pixels which have no width and no distance between
+       them -- only a sequence.  Instead, think of the raster row as a
+       bunch of pixels 1 unit wide adjacent to each other.  For
+       example, we are going to scale a 100 pixel row to a 150 pixel
+       row.  Imagine placing the input row right above the output row
+       and stretching it so it is the same size as the output row.  It
+       still contains 100 pixels, but they are 1.5 units wide each.
+       Our goal is to make the output row look as much as possible
+       like the input row, while observing that a pixel can only be
+       one color.
+
+       Output Pixel 0 is completely covered by Input Pixel 0, so we
+       make Output Pixel 0 the same color as Input Pixel 0.  Output
+       Pixel 1 is covered half by Input Pixel 0 and half by Input
+       Pixel 1.  So we make Output Pixel 1 a 50/50 mix of Input Pixels
+       0 and 1.  If you stand back far enough, input and output will
+       look the same.
+
+       This works for all scale factors, both scaling up and scaling down.
+       
+       This program always stretches or squeezes the input row to be the
+       same length as the output row; The output row's pixels are always
+       1 unit wide.
+
+       The same thing works in the vertical direction.  We think of
+       rows as stacked strips of 1 unit height.  We conceptually
+       stretch the image vertically first (same process as above, but
+       in place of a single-color pixels, we have a vector of colors).
+       Then we take each row this vertical stretching generates and
+       stretch it horizontally.  
+    */
+
+    xel* xelrow;  /* An input row */
+    xel* vertScaledRow;
+        /* An output row after vertical scaling, but before horizontal
+           scaling
+        */
+    xel* newxelrow;
+    float rowsleft;
+        /* The number of rows of output that need to be formed from the
+           current input row (the one in xelrow[]), less the number that 
+           have already been formed (either in the rs/gs/bs accumulators
+           or output to the file).  This can be fractional because of the
+           way we define rows as having height.
+        */
+    float fracrowtofill;
+        /* The fraction of the current output row (the one in vertScaledRow[])
+           that hasn't yet been filled in from an input row.
+        */
+    float *rs, *gs, *bs;
+        /* The red, green, and blue color intensities so far accumulated
+           from input rows for the current output row.
+        */
+    int rowsread;
+        /* Number of rows of the input file that have been read */
+    int row;
+    
+    xelrow = pnm_allocrow(cols); 
+    vertScaledRow = pnm_allocrow(cols);
+    rs = (float*) pm_allocrow( cols, sizeof(float) );
+    gs = (float*) pm_allocrow( cols, sizeof(float) );
+    bs = (float*) pm_allocrow( cols, sizeof(float) );
+    rowsread = 0;
+    rowsleft = 0.0;
+    zeroAccum(cols, format, rs, gs, bs);
+    fracrowtofill = 1.0;
+
+    newxelrow = pnm_allocrow( newcols );
+
+    for ( row = 0; row < newrows; ++row ) {
+        /* First scale Y from xelrow[] into vertScaledRow[]. */
+
+        if ( newrows == rows ) { /* shortcut Y scaling if possible */
+            pnm_readpnmrow( ifP, vertScaledRow, cols, newmaxval, format );
+	    } else {
+            while (fracrowtofill > 0) {
+                if (rowsleft <= 0.0) {
+                    if (rowsread < rows) {
+                        pnm_readpnmrow(ifP, xelrow, cols, newmaxval, format);
+                        ++rowsread;
+                    } else {
+                        /* We need another input row to fill up this
+                           output row, but there aren't any more.
+                           That's because of rounding down on our
+                           scaling arithmetic.  So we go ahead with
+                           the data from the last row we read, which
+                           amounts to stretching out the last output
+                           row.  
+                        */
+                        if (verbose)
+                            pm_message("%f of bottom row stretched due to "
+                                       "arithmetic imprecision", 
+                                       fracrowtofill);
+                    }
+                    rowsleft = yscale;
+                }
+                if (rowsleft < fracrowtofill) {
+                    accumOutputRow(xelrow, rowsleft, rs, gs, bs, 
+                                   cols, format);
+                    fracrowtofill -= rowsleft;
+                    rowsleft = 0.0;
+                }
+                else {
+                    accumOutputRow(xelrow, fracrowtofill, rs, gs, bs,
+                                   cols, format);
+                    rowsleft = rowsleft - fracrowtofill;
+                    fracrowtofill = 0.0;
+                }
+            }
+            makeRow(vertScaledRow, rs, gs, bs, cols, newmaxval, format);
+            zeroAccum(cols, format, rs, gs, bs);
+            fracrowtofill = 1.0;
+	    }
+
+        /* Now scale vertScaledRow horizontally into newxelrow and write
+           it out. 
+        */
+
+        if (newcols == cols)	/* shortcut X scaling if possible */
+            pnm_writepnmrow(stdout, vertScaledRow, newcols, 
+                            newmaxval, newformat, 0);
+        else {
+            float stretch;
+
+            horizontal_scale(vertScaledRow, newxelrow, cols, newcols, xscale, 
+                             format, newmaxval, &stretch);
+            
+            if (verbose && row == 0)
+                pm_message("%f of right column stretched due to "
+                           "arithmetic imprecision", 
+                           stretch);
+            
+            pnm_writepnmrow(stdout, newxelrow, newcols, 
+                            newmaxval, newformat, 0 );
+        }
+	}
+    pnm_freerow(newxelrow);
+    pnm_freerow(xelrow);
+    pnm_freerow(vertScaledRow);
+}
+
+
+
+static void
+scaleWithoutMixing(FILE * const ifP,
+                   int const cols, int const rows,
+                   xelval const maxval, int const format,
+                   int const newcols, int const newrows,
+                   xelval const newmaxval, int const newformat,
+                   float const xscale, float const yscale) {
+/*----------------------------------------------------------------------------
+   Scale the image on input file 'ifP' (which is described by 
+   'cols', 'rows', 'format', and 'maxval') by xscale horizontally and
+   yscale vertically and write the result to standard output as format
+   'newformat' and with maxval 'newmaxval'.
+
+   The input file is positioned past the header, to the beginning of the
+   raster.  The output file is too.
+
+   Don't mix colors from different input pixels together in the output
+   pixels.  Each output pixel is an exact copy of some corresponding 
+   input pixel.
+-----------------------------------------------------------------------------*/
+    xel* xelrow;  /* An input row */
+    xel* newxelrow;
+    int row;
+    int rowInXelrow;
+
+    xelrow = pnm_allocrow(cols); 
+    rowInXelrow = -1;
+
+    newxelrow = pnm_allocrow(newcols);
+
+    for (row = 0; row < newrows; ++row) {
+        int col;
+        
+        int const inputRow = (int) (row / yscale);
+
+        for (; rowInXelrow < inputRow; ++rowInXelrow) 
+            pnm_readpnmrow(ifP, xelrow, cols, newmaxval, format);
+        
+
+        for (col = 0; col < newcols; ++col) {
+            int const inputCol = (int) (col / xscale);
+            
+            newxelrow[col] = xelrow[inputCol];
+        }
+
+        pnm_writepnmrow(stdout, newxelrow, newcols, 
+                        newmaxval, newformat, 0 );
+	}
+    pnm_freerow(xelrow);
+    pnm_freerow(newxelrow);
+}
+
+
+
 int
 main(int argc, char **argv ) {
 
     struct cmdline_info cmdline;
-    FILE* ifp;
-    xel* xelrow;
-    xel* tempxelrow;
-    xel* newxelrow;
-    int rows, cols, format, newformat, rowsread, newrows, newcols;
-    int row, col, needtoreadrow;
+    FILE* ifP;
+    int rows, cols, format, newformat, newrows, newcols;
     xelval maxval, newmaxval;
     float xscale, yscale;
-    float fracrowtofill, fracrowleft;
-    float* rs;
-    float* gs;
-    float* bs;
 
     pnm_init( &argc, argv );
 
     parse_command_line(argc, argv, &cmdline);
 
-    ifp = pm_openr(cmdline.input_filespec);
+    ifP = pm_openr(cmdline.input_filespec);
 
-    pnm_readpnminit( ifp, &cols, &rows, &maxval, &format );
+    pnm_readpnminit( ifP, &cols, &rows, &maxval, &format );
 
     /* Promote PBM files to PGM. */
     if ( PNM_FORMAT_TYPE(format) == PBM_TYPE ) {
@@ -420,131 +710,19 @@ main(int argc, char **argv ) {
                  "do the specified scaling.  Use a smaller input image "
                  "or a slightly different scale factor.");
 
-    xelrow = pnm_allocrow(cols);
-    if (newrows == rows)	/* shortcut Y scaling if possible */
-        tempxelrow = xelrow;
+    pnm_writepnminit(stdout, newcols, newrows, newmaxval, newformat, 0);
+
+    if (cmdline.nomix) 
+        scaleWithoutMixing(ifP, cols, rows, maxval, format,
+                           newcols, newrows, newmaxval, newformat, 
+                           xscale, yscale);
     else
-        tempxelrow = pnm_allocrow( cols );
-    rs = (float*) pm_allocrow( cols, sizeof(float) );
-    gs = (float*) pm_allocrow( cols, sizeof(float) );
-    bs = (float*) pm_allocrow( cols, sizeof(float) );
-    rowsread = 0;
-    fracrowleft = yscale;
-    needtoreadrow = 1;
-    for ( col = 0; col < cols; ++col )
-        rs[col] = gs[col] = bs[col] = 0.0;
-    fracrowtofill = 1.0;
+        scaleWithMixing(ifP, cols, rows, maxval, format,
+                        newcols, newrows, newmaxval, newformat, 
+                        xscale, yscale, cmdline.verbose);
 
-    pnm_writepnminit( stdout, newcols, newrows, newmaxval, newformat, 0 );
-    newxelrow = pnm_allocrow( newcols );
-
-    for ( row = 0; row < newrows; ++row ) {
-        /* First scale Y from xelrow into tempxelrow. */
-        if ( newrows == rows ) { /* shortcut Y scaling if possible */
-            pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
-	    } else {
-            while ( fracrowleft < fracrowtofill ) {
-                if ( needtoreadrow )
-                    if ( rowsread < rows ) {
-                        pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
-                        ++rowsread;
-                    }
-                switch ( PNM_FORMAT_TYPE(format) ) {
-                case PPM_TYPE:
-                    for ( col = 0; col < cols; ++col ) {
-                        rs[col] += fracrowleft * PPM_GETR(xelrow[col]);
-                        gs[col] += fracrowleft * PPM_GETG(xelrow[col]);
-                        bs[col] += fracrowleft * PPM_GETB(xelrow[col]);
-                    }
-                    break;
-
-                default:
-                    for ( col = 0; col < cols; ++col)
-                        gs[col] += fracrowleft * PNM_GET1(xelrow[col]);
-                    break;
-                }
-                fracrowtofill -= fracrowleft;
-                fracrowleft = yscale;
-                needtoreadrow = 1;
-            }
-            /* Now fracrowleft is >= fracrowtofill, so we can produce a row. */
-            if ( needtoreadrow ) {
-                if ( rowsread < rows ) {
-                    pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
-                    ++rowsread;
-                    needtoreadrow = 0;
-                } else {
-                    /* We need another input row to fill up this output row,
-                       but there aren't any more.  That's because of rounding
-                       down on our scaling arithmetic.  So we go ahead with 
-                       the data from the last row we read, which amounts to 
-                       stretching out the last output row.
-                    */
-                    if (cmdline.verbose)
-                        pm_message("%f of bottom row stretched due to "
-                                   "arithmetic imprecision", 
-                                   fracrowtofill);
-                }
-            }
-            switch ( PNM_FORMAT_TYPE(format) ) {
-            case PPM_TYPE:
-                for ( col = 0; col < cols; ++col) {
-                    float r, g, b;
-
-                    r = rs[col] + fracrowtofill * PPM_GETR(xelrow[col]);
-                    g = gs[col] + fracrowtofill * PPM_GETG(xelrow[col]);
-                    b = bs[col] + fracrowtofill * PPM_GETB(xelrow[col]);
-                    PPM_ASSIGN( tempxelrow[col], 
-                        min(newmaxval, (int) (r + 0.5)), 
-                        min(newmaxval, (int) (g + 0.5)), 
-                        min(newmaxval, (int) (b + 0.5))
-                        );
-                    rs[col] = gs[col] = bs[col] = 0.0;
-                }
-                break;
-
-            default:
-                for ( col = 0; col < cols; ++col ) {
-                    float g;
-                    
-                    g = gs[col] + fracrowtofill * PNM_GET1(xelrow[col]);
-                    PNM_ASSIGN1(tempxelrow[col], 
-                                min(newmaxval, (int) (g + 0.5)));
-                    gs[col] = 0.0;
-                }
-                break;
-            }
-            fracrowleft -= fracrowtofill;
-            if ( fracrowleft <= 0.0 ) {
-                fracrowleft = yscale;
-                needtoreadrow = 1;
-            }
-            fracrowtofill = 1.0;
-	    }
-
-        /* Now scale tempxelrow horizontally into newxelrow & write it out. */
-
-        if (newcols == cols)	/* shortcut X scaling if possible */
-            pnm_writepnmrow(stdout, tempxelrow, newcols, 
-                            newmaxval, newformat, 0);
-        else {
-            float stretch;
-
-            horizontal_scale(tempxelrow, newxelrow, cols, newcols, xscale, 
-                             format, newmaxval, &stretch);
-            
-            if (cmdline.verbose && row == 0)
-                pm_message("%f of right column stretched due to "
-                           "arithmetic imprecision", 
-                           stretch);
-            
-            pnm_writepnmrow(stdout, newxelrow, newcols, 
-                            newmaxval, newformat, 0 );
-        }
-	}
+    pm_close(ifP);
+    pm_close(stdout);
     
-    pm_close( ifp );
-    pm_close( stdout );
-    
-    exit( 0 );
+    exit(0);
 }
