@@ -8,8 +8,17 @@
 ** copyright notice and this permission notice appear in supporting
 ** documentation.  This software is provided "as is" without express or
 ** implied warranty.
+**
+** Modified:
+**
+** June 6, 2001: Christopher W. Boyd <cboyd@pobox.com>
+**               - added -reduce N to allow scaling by integer value
+**                 in this case, scale_comp becomes 1/N and x/yscale
+**                 get set as they should
+**    
+**
 */
-
+ 
 #include <math.h>
 #include "pnm.h"
 
@@ -56,6 +65,7 @@ parse_command_line(int argc, char ** argv,
 
     unsigned int option_def_index;
     int xysize, xsize, ysize, pixels;
+    int reduce;
     float xscale, yscale, scale_parm;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
@@ -68,6 +78,7 @@ parse_command_line(int argc, char ** argv,
     OPTENTRY(0,   "pixels",    OPT_UINT,    &pixels,        0);
     OPTENTRY(0,   "xysize",    OPT_FLAG,    &xysize,        0);
     OPTENTRY(0,   "verbose",   OPT_FLAG,    &cmdline_p->verbose,        0);
+    OPTENTRY(0,   "reduce",    OPT_UINT,    &reduce,        0);
 
     /* Set the defaults. -1 = unspecified */
     xsize = -1;
@@ -76,6 +87,8 @@ parse_command_line(int argc, char ** argv,
     yscale = -1.0;
     pixels = -1;
     xysize = 0;
+    reduce = -1;
+    cmdline_p->verbose = FALSE;
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -92,6 +105,8 @@ parse_command_line(int argc, char ** argv,
         pm_error("-xscale must be greater than zero.");
     if (yscale != -1.0 && yscale <= 0.0)
         pm_error("-yscale must be greater than zero.");
+    if (reduce <= 0 && reduce != -1)
+        pm_error("-reduce must be greater than zero.");
 
     if (xsize != -1 && xscale != -1)
         pm_error("Cannot specify both -xsize/width and -xscale.");
@@ -99,11 +114,16 @@ parse_command_line(int argc, char ** argv,
         pm_error("Cannot specify both -ysize/height and -yscale.");
     
     if (xysize && 
-        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1) )
+        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1 || 
+         reduce != -1 || pixels != -1) )
         pm_error("Cannot specify -xysize with other dimension options.");
     if (pixels != -1 && 
-        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1) )
+        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1 ||
+         reduce != -1) )
         pm_error("Cannot specify -pixels with other dimension options.");
+    if (reduce != -1 && 
+        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1) )
+        pm_error("Cannot specify -reduce with other dimension options.");
 
     if (pixels == 0)
         pm_error("-pixels must be greater than zero");
@@ -133,7 +153,7 @@ parse_command_line(int argc, char ** argv,
         cmdline_p->ybox = 0;
         
         if (xsize == -1 && xscale == -1 && ysize == -1 && yscale == -1
-            && pixels == -1) {
+            && pixels == -1 && reduce == -1) {
             /* parameters are scale factor and optional filespec */
             if (argc-1 < 1)
                 pm_error("With no dimension options, you must supply at least "
@@ -154,11 +174,17 @@ parse_command_line(int argc, char ** argv,
             }
         } else {
             /* Only parameter allowed is optional filespec */
-            scale_parm = 0.0;
             if (argc-1 < 1)
                 cmdline_p->input_filespec = "-";
             else
                 cmdline_p->input_filespec = argv[1];
+
+            if (reduce != -1) {
+                scale_parm = ((double) 1.0) / ((double) reduce);
+                pm_message("reducing by %d gives scale factor of %f.", 
+                           reduce, scale_parm);
+            } else
+                scale_parm = 0.0;
         }
     }
 
@@ -377,7 +403,7 @@ main(int argc, char **argv ) {
     xel* nxP;
     int rows, cols, format, newformat, rowsread, newrows, newcols;
     int row, col, needtoreadrow;
-    xelval maxval;
+    xelval maxval, newmaxval;
     long sxscale, syscale;
     long fracrowtofill, fracrowleft;
     long* rs;
@@ -395,18 +421,17 @@ main(int argc, char **argv ) {
 
     ifp = pm_openr(cmdline.input_filespec);
 
-    pnm_pbmmaxval = PNM_MAXMAXVAL;  /* use larger value for better results */
     pnm_readpnminit( ifp, &cols, &rows, &maxval, &format );
 
     /* Promote PBM files to PGM. */
-    if ( PNM_FORMAT_TYPE(format) == PBM_TYPE )
-	{
+    if ( PNM_FORMAT_TYPE(format) == PBM_TYPE ) {
         newformat = PGM_TYPE;
-	pm_message( "promoting from PBM to PGM" );
-	}
-    else
+        newmaxval = PGM_MAXMAXVAL;
+        pm_message( "promoting from PBM to PGM" );
+	}  else {
         newformat = format;
-
+        newmaxval = maxval;
+    }
     compute_output_dimensions(cmdline, rows, cols, &newrows, &newcols);
 
     /* We round the scale factor down so that we never fill up the
@@ -419,9 +444,9 @@ main(int argc, char **argv ) {
     syscale = SCALE * newrows / rows;
 
     if (cmdline.verbose) {
-        pm_message("Scaling by %d/%d = %f horizontally to %d columns.", 
+        pm_message("Scaling by %ld/%d = %f horizontally to %d columns.", 
                    sxscale, SCALE, (float) sxscale/SCALE, newcols );
-        pm_message("Scaling by %d/%d = %f vertically to %d rows.", 
+        pm_message("Scaling by %ld/%d = %f vertically to %d rows.", 
                    syscale, SCALE, (float) syscale/SCALE, newrows);
     }
 
@@ -441,18 +466,18 @@ main(int argc, char **argv ) {
     fracrowtofill = SCALE;
     vertical_stretch = 0;
     
-    pnm_writepnminit( stdout, newcols, newrows, maxval, newformat, 0 );
+    pnm_writepnminit( stdout, newcols, newrows, newmaxval, newformat, 0 );
     newxelrow = pnm_allocrow( newcols );
     
     for ( row = 0; row < newrows; ++row ) {
         /* First scale vertically from xelrow into tempxelrow. */
         if ( newrows == rows ) { /* shortcut vertical scaling if possible */
-            pnm_readpnmrow( ifp, xelrow, cols, maxval, format );
+            pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
 	    } else {
             while ( fracrowleft < fracrowtofill ) {
                 if ( needtoreadrow )
                     if ( rowsread < rows ) {
-                        pnm_readpnmrow( ifp, xelrow, cols, maxval, format );
+                        pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
                         ++rowsread;
                     }
                 switch ( PNM_FORMAT_TYPE(format) ) {
@@ -476,7 +501,7 @@ main(int argc, char **argv ) {
             /* Now fracrowleft is >= fracrowtofill, so we can produce a row. */
             if ( needtoreadrow ) {
                 if ( rowsread < rows ) {
-                    pnm_readpnmrow( ifp, xelrow, cols, maxval, format );
+                    pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
                     ++rowsread;
                     needtoreadrow = 0;
                 } else {
@@ -499,11 +524,11 @@ main(int argc, char **argv ) {
                     g = gs[col] + fracrowtofill * PPM_GETG( *xP );
                     b = bs[col] + fracrowtofill * PPM_GETB( *xP );
                     r /= SCALE;
-                    if ( r > maxval ) r = maxval;
+                    if ( r > newmaxval ) r = newmaxval;
                     g /= SCALE;
-                    if ( g > maxval ) g = maxval;
+                    if ( g > newmaxval ) g = newmaxval;
                     b /= SCALE;
-                    if ( b > maxval ) b = maxval;
+                    if ( b > newmaxval ) b = newmaxval;
                     PPM_ASSIGN( *nxP, r, g, b );
                     rs[col] = gs[col] = bs[col] = HALFSCALE;
                 }
@@ -516,7 +541,7 @@ main(int argc, char **argv ) {
                     
                     g = gs[col] + fracrowtofill * PNM_GET1( *xP );
                     g /= SCALE;
-                    if ( g > maxval ) g = maxval;
+                    if ( g > newmaxval ) g = newmaxval;
                     PNM_ASSIGN1( *nxP, g );
                     gs[col] = HALFSCALE;
                 }
@@ -533,26 +558,28 @@ main(int argc, char **argv ) {
         /* Now scale tempxelrow horizontally into newxelrow & write it out. */
 
         if (newcols == cols)	/* shortcut X scaling if possible */
-            pnm_writepnmrow(stdout, tempxelrow, newcols, maxval, newformat, 0);
+            pnm_writepnmrow(stdout, tempxelrow, newcols, 
+                            newmaxval, newformat, 0);
         else {
             int stretch;
 
             horizontal_scale(tempxelrow, newxelrow, cols, newcols, sxscale, 
-                             format, maxval, &stretch);
+                             format, newmaxval, &stretch);
             
             if (cmdline.verbose && row == 0 && stretch != 0)
                 pm_message("%d/%d = %f right columns filled by stretching "
                            "due to arithmetic imprecision", 
                            stretch, SCALE, (float) stretch/SCALE);
             
-            pnm_writepnmrow(stdout, newxelrow, newcols, maxval, newformat, 0 );
+            pnm_writepnmrow(stdout, newxelrow, newcols, 
+                            newmaxval, newformat, 0 );
         }
 	}
 
     if (cmdline.verbose && vertical_stretch != 0)
         pm_message("%d/%d = %f bottom rows filled by stretching due to "
                    "arithmetic imprecision", 
-                   fracrowtofill, SCALE, 
+                   vertical_stretch, SCALE, 
                    (float) vertical_stretch/SCALE);
     
     pm_close( ifp );
