@@ -49,7 +49,9 @@ struct cmdline_info {
     int g3options;     /* G3OPTIONS Tiff tag value or 0 for none */
     int predictor;     /* PREDICTOR Tiff tag value or -1 for none */
     int rowsperstrip;  /* -1 = unspecified */
-    int minisblack;    /* logical: User wants MINISBLACK photometric */
+    unsigned int minisblack;   /* logical: User wants MINISBLACK photometric */
+    unsigned int truecolor;    /* logical: User wants truecolor, not cmap */
+    unsigned int color;        /* logical: assume not grayscale */
     float xresolution; /* XRESOLUTION Tiff tag value or -1 for none */
     float yresolution; /* YRESOLUTION Tiff tag value or -1 for none */
 };
@@ -63,39 +65,37 @@ parse_command_line(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
-         */
-    optStruct2 opt;
+    optEntry *option_def = malloc(100*sizeof(optEntry));
 
-    int none, packbits, lzw, g3, g4, msb2lsb, lsb2msb, opt_2d, fill;
-    int flate, adobeflate;
+    optStruct3 opt;
+
+    unsigned int none, packbits, lzw, g3, g4, msb2lsb, lsb2msb, opt_2d, fill;
+    unsigned int flate, adobeflate;
     
     unsigned int option_def_index;
 
-    option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0, "none",         OPT_FLAG,   &none,                    0);
-    OPTENTRY(0, "packbits",     OPT_FLAG,   &packbits,                0);
-    OPTENTRY(0, "lzw",          OPT_FLAG,   &lzw,                     0);
-    OPTENTRY(0, "g3",           OPT_FLAG,   &g3,                      0);
-    OPTENTRY(0, "g4",           OPT_FLAG,   &g4,                      0);
-    OPTENTRY(0, "flate",        OPT_FLAG,   &flate,                   0);
-    OPTENTRY(0, "adobeflate",   OPT_FLAG,   &adobeflate,              0);
-    OPTENTRY(0, "msb2lsb",      OPT_FLAG,   &msb2lsb,                 0);
-    OPTENTRY(0, "lsb2msb",      OPT_FLAG,   &lsb2msb,                 0);
-    OPTENTRY(0, "2d",           OPT_FLAG,   &opt_2d,                  0);
-    OPTENTRY(0, "fill",         OPT_FLAG,   &fill,                    0);
-    OPTENTRY(0, "minisblack",   OPT_FLAG,   &cmdline_p->minisblack,   0);
-    OPTENTRY(0, "mb",           OPT_FLAG,   &cmdline_p->minisblack,   0);
-    OPTENTRY(0, "predictor",    OPT_UINT,   &cmdline_p->predictor,    0);
-    OPTENTRY(0, "rowsperstrip", OPT_UINT,   &cmdline_p->rowsperstrip, 0);
-    OPTENTRY(0, "xresolution",  OPT_FLOAT,  &cmdline_p->xresolution,  0);
-    OPTENTRY(0, "yresolution",  OPT_FLOAT,  &cmdline_p->yresolution,  0);
+    option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0, "none",         OPT_FLAG,   NULL, &none,                    0);
+    OPTENT3(0, "packbits",     OPT_FLAG,   NULL, &packbits,                0);
+    OPTENT3(0, "lzw",          OPT_FLAG,   NULL, &lzw,                     0);
+    OPTENT3(0, "g3",           OPT_FLAG,   NULL, &g3,                      0);
+    OPTENT3(0, "g4",           OPT_FLAG,   NULL, &g4,                      0);
+    OPTENT3(0, "flate",        OPT_FLAG,   NULL, &flate,                   0);
+    OPTENT3(0, "adobeflate",   OPT_FLAG,   NULL, &adobeflate,              0);
+    OPTENT3(0, "msb2lsb",      OPT_FLAG,   NULL, &msb2lsb,                 0);
+    OPTENT3(0, "lsb2msb",      OPT_FLAG,   NULL, &lsb2msb,                 0);
+    OPTENT3(0, "2d",           OPT_FLAG,   NULL, &opt_2d,                  0);
+    OPTENT3(0, "fill",         OPT_FLAG,   NULL, &fill,                    0);
+    OPTENT3(0, "minisblack",   OPT_FLAG,   NULL, &cmdline_p->minisblack,   0);
+    OPTENT3(0, "mb",           OPT_FLAG,   NULL, &cmdline_p->minisblack,   0);
+    OPTENT3(0, "truecolor",    OPT_FLAG,   NULL, &cmdline_p->truecolor,    0);
+    OPTENT3(0, "color",        OPT_FLAG,   NULL, &cmdline_p->color,        0);
+    OPTENT3(0, "predictor",    OPT_UINT,   &cmdline_p->predictor,    NULL, 0);
+    OPTENT3(0, "rowsperstrip", OPT_UINT,   &cmdline_p->rowsperstrip, NULL, 0);
+    OPTENT3(0, "xresolution",  OPT_FLOAT,  &cmdline_p->xresolution,  NULL, 0);
+    OPTENT3(0, "yresolution",  OPT_FLOAT,  &cmdline_p->yresolution,  NULL, 0);
 
     /* Set the defaults */
-    none = packbits = lzw = g3 = g4 = msb2lsb = lsb2msb = opt_2d = FALSE;
-    flate = adobeflate = FALSE;
-    fill = FALSE;
     cmdline_p->predictor = -1;
     cmdline_p->rowsperstrip = -1;
     cmdline_p->minisblack = FALSE;
@@ -106,7 +106,7 @@ parse_command_line(int argc, char ** argv,
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    pm_optParseOptions2(&argc, argv, opt, 0);
+    optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
     if (none + packbits + lzw + g3 + g4 + flate + adobeflate > 1)
@@ -158,11 +158,11 @@ parse_command_line(int argc, char ** argv,
                  cmdline_p->rowsperstrip);
 
     if (cmdline_p->xresolution != -1 && cmdline_p->xresolution < 1)
-        pm_error("-xresolution must be positive.  You specified %d.",
+        pm_error("-xresolution must be positive.  You specified %f.",
                  cmdline_p->xresolution);
 
     if (cmdline_p->yresolution != -1 && cmdline_p->yresolution < 1)
-        pm_error("-yresolution must be positive.  You specified %d.",
+        pm_error("-yresolution must be positive.  You specified %f.",
                  cmdline_p->yresolution);
 
     if (argc-1 == 0) 
@@ -251,7 +251,7 @@ fillRowOfSubBytePixels(const xel * const xelrow, unsigned char* const buf,
 
 
 static void
-writeScanLines(const xel **    const xels, 
+writeScanLines(FILE *          const ifp,
                TIFF *          const tif, 
                colorhash_table const cht,
                int             const format, 
@@ -265,8 +265,10 @@ writeScanLines(const xel **    const xels,
                bool            const grayscale,
                int             const fillorder) {
 /*----------------------------------------------------------------------------
-   Write out the raster 'xels'.
+   Write out the raster for the image in file 'ifp', which is positioned
+   to the header of the image.
 -----------------------------------------------------------------------------*/
+    xel * xelrow;  /* malloc'ed */
     unsigned char* buf;
     int row;
 
@@ -287,11 +289,15 @@ writeScanLines(const xel **    const xels,
 
     buf = (unsigned char*) malloc( bytesperrow );
     if ( buf == (unsigned char*) 0 )
-	pm_error( "can't allocate memory for row buffer" );
+    pm_error( "can't allocate memory for row buffer" );
 
+    xelrow = pnm_allocrow(cols);
+    
     for (row = 0; row < rows; ++row) {
         int col;
 
+        pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
+
         if ( PNM_FORMAT_TYPE(format) == PPM_TYPE && ! grayscale ) {
             if (cht == NULL) {
                 /* It's an RGB raster */
@@ -299,11 +305,11 @@ writeScanLines(const xel **    const xels,
                 for (col = 0, tP = buf; col < cols; ++col) {
                     register xelval s;
 
-                    s = PPM_GETR( xels[row][col] );
+                    s = PPM_GETR( xelrow[col] );
                     PUTSAMPLE;  /* advances tP */
-                    s = PPM_GETG( xels[row][col] );
+                    s = PPM_GETG( xelrow[col] );
                     PUTSAMPLE;  /* advances tP */
-                    s = PPM_GETB( xels[row][col] );
+                    s = PPM_GETB( xelrow[col] );
                     PUTSAMPLE;  /* advances tP */
                 }
             } else {
@@ -311,13 +317,13 @@ writeScanLines(const xel **    const xels,
                 for (col = 0; col < cols; ++col) {
                     int s;
 
-                    s = ppm_lookupcolor(cht, &xels[row][col]);
+                    s = ppm_lookupcolor(cht, &xelrow[col]);
                     if (s == -1)
                         pm_error("color not found?!?  "
                                  "row=%d col=%d  r=%d g=%d b=%d",
-                                 row, col, PPM_GETR( xels[row][col] ), 
-                                 PPM_GETG(xels[row][col]),
-                                 PPM_GETB(xels[row][col]));
+                                 row, col, PPM_GETR( xelrow[col] ), 
+                                 PPM_GETG(xelrow[col]),
+                                 PPM_GETB(xelrow[col]));
                     buf[col] = (unsigned char) s;
                 }
             }
@@ -327,17 +333,167 @@ writeScanLines(const xel **    const xels,
                 unsigned char* tP;
                 tP = buf;
                 for (col = 0; col < cols; ++col) {
-                    xelval s = PNM_GET1(xels[row][col]);
+                    xelval s = PNM_GET1(xelrow[col]);
                     PUTSAMPLE; /* advances tP */
                 }
             } else 
-                fillRowOfSubBytePixels(xels[row], buf, maxval, tiff_maxval,
+                fillRowOfSubBytePixels(xelrow, buf, maxval, tiff_maxval,
                                        photometric,
                                        fillorder, bitspersample, cols);
         }
         if (TIFFWriteScanline( tif, buf, row, 0 ) < 0)
             pm_error( "failed a scanline write on row %d", row );
     }
+    pnm_freerow(xelrow);
+    free(buf);
+}
+
+
+
+static void
+analyzeColors(FILE *              const ifp,
+              struct cmdline_info const cmdline,
+              int                 const cols,
+              int                 const rows,
+              xelval              const maxval,
+              int                 const format,
+              int                 const maxcolors, 
+              colorhist_vector *  const chvP, 
+              int *               const colorsP, 
+              bool *              const grayscaleP) {
+/*----------------------------------------------------------------------------
+   Analyze the colors in the image that is in the file 'ifp', which is
+   positioned to the raster. 
+
+   Leave the file position undefined.
+-----------------------------------------------------------------------------*/
+    switch (PNM_FORMAT_TYPE(format)) {
+    case PPM_TYPE:
+        if (cmdline.color && cmdline.truecolor) {
+            *chvP = NULL;
+            *grayscaleP = FALSE;
+        } else {
+            colorhist_vector chv;
+            bool grayscale;
+
+            pm_message( "computing colormap..." );
+            chv = ppm_computecolorhist2(ifp, cols, rows, maxval, format, 
+                                        maxcolors, colorsP);
+            if (chv == NULL) {
+                grayscale = FALSE;
+            } else {
+                int i;
+                pm_message("%d colors found", *colorsP);
+                grayscale = TRUE;  /* initial assumption */
+                for (i = 0; i < *colorsP && grayscale; ++i) {
+                    xelval r, g, b;
+
+                    r = PPM_GETR(chv[i].color);
+                    g = PPM_GETG(chv[i].color);
+                    b = PPM_GETB(chv[i].color);
+                    if (r != g || g != b) 
+                        grayscale = FALSE;
+                }
+            }
+            *grayscaleP = grayscale;
+
+            if (!chv && !cmdline.truecolor) {
+                pm_message("Too many colors - "
+                           "proceeding to write a 24-bit RGB file.");
+                pm_message("If you want an 8-bit palette file, "
+                           "try doing a 'ppmquant %d'.",
+                           maxcolors);
+                *chvP = NULL;
+            } else if (chv && cmdline.truecolor) {
+                free(chv);
+                *chvP = NULL;
+            } else 
+                *chvP = chv;
+        }
+        break;
+
+    default:
+        *chvP = NULL;
+        *grayscaleP = TRUE;
+        break;
+    }
+}
+
+
+
+static void
+computeRasterParm(int              const cols,
+                  int              const format, 
+                  xelval           const maxval, 
+                  colorhist_vector const chv, 
+                  bool             const grayscale,
+                  int              const compression,
+                  bool             const minisblack,
+                  int              const requested_rowsperstrip,
+                  unsigned short * const samplesperpixelP,
+                  unsigned short * const bitspersampleP,
+                  unsigned short * const photometricP,
+                  int            * const bytesperrowP,
+                  unsigned short * const rowsperstripP) {
+
+    /* We determine the bits per sample by the maxval, except that if
+       it would be more than 8, we just use 16.  I don't know if bits
+       per sample between 8 and 16 are legal, but they aren't very
+       nice in any case.  If users want them, we should provide an
+       option.  It is not clear why we don't use bits per pixel < 8
+       for RGB images.  Note that code to handle maxvals <= 255 was
+       written long before maxval > 255 was possible and there are
+       backward compatibility requirements.  
+    */
+    switch ( PNM_FORMAT_TYPE(format) )
+    {
+    case PPM_TYPE:
+        if (chv == NULL) {
+            *samplesperpixelP = 3;
+            *bitspersampleP = maxval > 255 ? 16 : 8;
+            *photometricP = PHOTOMETRIC_RGB;
+        } else if (grayscale) {
+            *samplesperpixelP = 1;
+            *bitspersampleP = maxval > 255 ? 16 : pm_maxvaltobits(maxval);
+            *photometricP = PHOTOMETRIC_MINISBLACK;
+        } else {
+            *samplesperpixelP = 1;
+            *bitspersampleP = 8;
+            *photometricP = PHOTOMETRIC_PALETTE;
+        }
+        break;
+
+    case PGM_TYPE:
+        *samplesperpixelP = 1;
+        *bitspersampleP = maxval > 255 ? 16 : pm_maxvaltobits(maxval);
+        *photometricP = PHOTOMETRIC_MINISBLACK;
+        break;
+
+    default:
+        *samplesperpixelP = 1;
+        *bitspersampleP = 1;
+        if (((compression == COMPRESSION_CCITTFAX3) ||
+             (compression == COMPRESSION_CCITTFAX4)) &&
+            ! minisblack)
+            *photometricP = PHOTOMETRIC_MINISWHITE;
+        else
+            *photometricP = PHOTOMETRIC_MINISBLACK;
+        break;
+    }
+
+    if (*bitspersampleP < 8) {
+        int samplesperbyte;
+        samplesperbyte = 8 / *bitspersampleP;
+        *bytesperrowP = 
+            (cols * *samplesperpixelP + samplesperbyte-1) / samplesperbyte;
+    } else 
+        *bytesperrowP = (cols * *samplesperpixelP * *bitspersampleP) / 8;
+
+    if (requested_rowsperstrip == -1 )
+        *rowsperstripP = (8 * 1024) / *bytesperrowP;
+    else 
+        *rowsperstripP = requested_rowsperstrip;
+
 }
 
 
@@ -347,7 +503,6 @@ main(int argc, char *argv[]) {
     struct cmdline_info cmdline;
     char* inf;
     FILE* ifp;
-    const xel** xels;
     colorhist_vector chv;
     colorhash_table cht;
     unsigned short red[MAXCOLORS], grn[MAXCOLORS], blu[MAXCOLORS];
@@ -364,61 +519,27 @@ main(int argc, char *argv[]) {
           determined solely by the bits per sample ('bitspersample').
           */
     int bytesperrow;
+    unsigned int rasterPos;
 
     pnm_init( &argc, argv );
 
     parse_command_line(argc, argv, &cmdline);
     
-    ifp = pm_openr(cmdline.input_filespec);
+    ifp = pm_openr_seekable(cmdline.input_filespec);
 
     if (strcmp(cmdline.input_filespec, "-") == 0)
         inf = "Standard Input";
     else 
         inf = cmdline.input_filespec;
 
-    xels = (const xel **) pnm_readpnm( ifp, &cols, &rows, &maxval, &format );
-    pm_close( ifp );
+    pnm_readpnminit(ifp, &cols, &rows, &maxval, &format);
 
-    /* Check for grayscale. */
-    switch ( PNM_FORMAT_TYPE(format) )
-	{
-	case PPM_TYPE:
-	pm_message( "computing colormap..." );
-	chv = ppm_computecolorhist((xel **) xels, cols, rows, MAXCOLORS, &colors );
-	if ( chv == (colorhist_vector) 0 )
-	    {
-	    pm_message(
-		"Too many colors - proceeding to write a 24-bit RGB file." );
-	    pm_message(
-		"If you want an 8-bit palette file, try doing a 'ppmquant %d'.",
-		MAXCOLORS );
-	    grayscale = 0;
-	    }
-	else
-	    {
-	    pm_message( "%d colors found", colors );
-	    grayscale = 1;
-	    for ( i = 0; i < colors; ++i )
-		{
-		register xelval r, g, b;
+    rasterPos = pm_tell(ifp);
 
-		r = PPM_GETR( chv[i].color );
-		g = PPM_GETG( chv[i].color );
-		b = PPM_GETB( chv[i].color );
-		if ( r != g || g != b )
-		    {
-		    grayscale = 0;
-		    break;
-		    }
-		}
-	    }
-	break;
+    analyzeColors(ifp, cmdline, cols, rows, maxval, format, MAXCOLORS, 
+                  &chv, &colors, &grayscale);
 
-	default:
-	chv = (colorhist_vector) 0;
-	grayscale = 1;
-	break;
-	}
+    pm_seek(ifp, rasterPos);  /* Go back to beginning of raster */
 
     /* Open output file. */
     fcntl( 1, F_SETFL, O_NONBLOCK ) ; 
@@ -427,73 +548,18 @@ main(int argc, char *argv[]) {
          */
     tif = TIFFFdOpen( 1, "Standard Output", "w" );
     if ( tif == NULL )
-	pm_error( "error opening standard output as TIFF file" );
+        pm_error( "error opening standard output as TIFF file" );
 
     /* Figure out TIFF parameters. */
 
-    /* We determine the bits per sample by the maxval, except that if
-       it would be more than 8, we just use 16.  I don't know if bits
-       per sample between 8 and 16 are legal, but they aren't very
-       nice in any case.  If users want them, we should provide an
-       option.  It is not clear why we don't use bits per pixel < 8
-       for RGB images.  Note that code to handle maxvals <= 255 was
-       written long before maxval > 255 was possible and there are
-       backward compatibility requirements.  
-    */
-    switch ( PNM_FORMAT_TYPE(format) )
-	{
-	case PPM_TYPE:
-	if ( chv == (colorhist_vector) 0 )
-	    {
-	    samplesperpixel = 3;
-	    bitspersample = maxval > 255 ? 16 : 8;
-	    photometric = PHOTOMETRIC_RGB;
-	    }
-	else if ( grayscale )
-	    {
-	    samplesperpixel = 1;
-	    bitspersample = maxval > 255 ? 16 : pm_maxvaltobits( maxval );
-	    photometric = PHOTOMETRIC_MINISBLACK;
-	    }
-	else
-	    {
-	    samplesperpixel = 1;
-	    bitspersample = 8;
-	    photometric = PHOTOMETRIC_PALETTE;
-	    }
-	break;
+    computeRasterParm(cols, format, maxval, chv, grayscale, 
+                      cmdline.compression,
+                      cmdline.minisblack, cmdline.rowsperstrip,
+                      &samplesperpixel, &bitspersample, &photometric,
+                      &bytesperrow, &rowsperstrip);
 
-	case PGM_TYPE:
-	samplesperpixel = 1;
-	bitspersample = maxval > 255 ? 16 : pm_maxvaltobits( maxval );
-	photometric = PHOTOMETRIC_MINISBLACK;
-	break;
-
-	default:
-	samplesperpixel = 1;
-    bitspersample = 1;
-    if (((cmdline.compression == COMPRESSION_CCITTFAX3) ||
-         (cmdline.compression == COMPRESSION_CCITTFAX4)) &&
-        ! cmdline.minisblack)
-        photometric = PHOTOMETRIC_MINISWHITE;
-    else
-        photometric = PHOTOMETRIC_MINISBLACK;
-    break;
-	}
-    if (bitspersample < 8) {
-        int samplesperbyte;
-        samplesperbyte = 8 / bitspersample;
-        bytesperrow = 
-            (cols * samplesperpixel + samplesperbyte-1) / samplesperbyte;
-    } else 
-        bytesperrow = (cols * samplesperpixel * bitspersample) / 8;
     tiff_maxval = pm_bitstomaxval( bitspersample );
 
-    if ( cmdline.rowsperstrip == -1 )
-        rowsperstrip = ( 8 * 1024 ) / bytesperrow;
-    else 
-        rowsperstrip = cmdline.rowsperstrip;
-
     /* Set TIFF parameters. */
     TIFFSetField( tif, TIFFTAG_IMAGEWIDTH, cols );
     TIFFSetField( tif, TIFFTAG_IMAGELENGTH, rows );
@@ -511,38 +577,38 @@ main(int argc, char *argv[]) {
     TIFFSetField( tif, TIFFTAG_SAMPLESPERPIXEL, samplesperpixel );
     TIFFSetField( tif, TIFFTAG_ROWSPERSTRIP, rowsperstrip );
     if ( cmdline.xresolution != -1 )
-		TIFFSetField( tif, TIFFTAG_XRESOLUTION, cmdline.xresolution );
+        TIFFSetField( tif, TIFFTAG_XRESOLUTION, cmdline.xresolution );
     if ( cmdline.yresolution != -1 )
-		TIFFSetField( tif, TIFFTAG_YRESOLUTION, cmdline.yresolution );
+        TIFFSetField( tif, TIFFTAG_YRESOLUTION, cmdline.yresolution );
     TIFFSetField( tif, TIFFTAG_RESOLUTIONUNIT, RESUNIT_INCH);
     /* TIFFSetField( tif, TIFFTAG_STRIPBYTECOUNTS, rows / rowsperstrip ); */
     TIFFSetField( tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
 
-    if ( chv == (colorhist_vector) 0 )
-	cht = (colorhash_table) 0;
-    else
-	{
-	/* Make TIFF colormap. */
-	for ( i = 0; i < colors; ++i )
-	    {
-	    red[i] = (long) PPM_GETR( chv[i].color ) * 65535L / maxval;
-	    grn[i] = (long) PPM_GETG( chv[i].color ) * 65535L / maxval;
-	    blu[i] = (long) PPM_GETB( chv[i].color ) * 65535L / maxval;
-	    }
-	TIFFSetField( tif, TIFFTAG_COLORMAP, red, grn, blu );
+    if (!chv)
+        cht = NULL;
+    else {
+        /* Make TIFF colormap. */
+        for (i = 0; i < colors; ++i) {
+            red[i] = (long) PPM_GETR( chv[i].color ) * 65535L / maxval;
+            grn[i] = (long) PPM_GETG( chv[i].color ) * 65535L / maxval;
+            blu[i] = (long) PPM_GETB( chv[i].color ) * 65535L / maxval;
+        }
+        TIFFSetField( tif, TIFFTAG_COLORMAP, red, grn, blu );
 
-	/* Convert color vector to color hash table, for fast lookup. */
-	cht = ppm_colorhisttocolorhash( chv, colors );
-	ppm_freecolorhist( chv );
-	}
+        /* Convert color vector to color hash table, for fast lookup. */
+        cht = ppm_colorhisttocolorhash( chv, colors );
+        ppm_freecolorhist( chv );
+    }
 
-    writeScanLines(xels, tif, cht, format, cols, rows, maxval, 
+    writeScanLines(ifp, tif, cht, format, cols, rows, maxval, 
                    tiff_maxval, bitspersample, photometric, bytesperrow, 
                    grayscale, cmdline.fillorder);
 
-    TIFFFlushData( tif );
-    TIFFClose( tif );
+    TIFFFlushData(tif);
+    TIFFClose(tif);
 
-    exit( 0 );
-    }
+    pm_close(ifp);
+
+    exit(0);
+}
 
