@@ -63,25 +63,25 @@
 #define PHOTOMETRIC_DEPTH 32768
 #endif
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    char *input_filename;
+    char *inputFilename;
     unsigned int headerdump;
-    unsigned int respectfillorder;
-    char *alpha_filename;
-    int alpha_stdout;
+    char *alphaFilename;
+    int alphaStdout;
+    int respectfillorder;   /* -respectfillorder option */
 } cmdline;
 
 
 
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo *cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that many of the strings that this function returns in the
-   *cmdline_p structure are actually in the supplied argv array.  And
+   *cmdlineP structure are actually in the supplied argv array.  And
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 -----------------------------------------------------------------------------*/
@@ -95,27 +95,27 @@ parse_command_line(int argc, char ** argv,
     opt.allowNegNum = FALSE;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
+    OPTENT3(0, "respectfillorder", 
+            OPT_FLAG,   NULL, &cmdlineP->headerdump, 0);
     OPTENT3('h', "headerdump", 
-            OPT_FLAG,   NULL, &cmdline_p->headerdump, 0);
+            OPT_FLAG,   NULL, &cmdlineP->headerdump, 0);
     OPTENT3(0,   "alphaout",   
-            OPT_STRING, &cmdline_p->alpha_filename, &alpha_spec, 0);
-    OPTENT3('h', "respectfillorder", 
-            OPT_FLAG,   NULL, &cmdline_p->respectfillorder, 0);
+            OPT_STRING, &cmdlineP->alphaFilename, &alpha_spec, 0);
 
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
 
     if (argc - 1 == 0)
-        cmdline_p->input_filename = NULL;  /* he wants stdin */
+        cmdlineP->inputFilename = NULL;  /* he wants stdin */
     else if (argc - 1 == 1)
-        cmdline_p->input_filename = strdup(argv[1]);
+        cmdlineP->inputFilename = strdup(argv[1]);
     else 
         pm_error("Too many arguments.  The only argument accepted\n"
                  "is the input file specificaton");
 
-    if (alpha_spec && strcmp(cmdline_p->alpha_filename, "-") == 0 )
-        cmdline_p->alpha_stdout = 1;
+    if (alpha_spec && strcmp(cmdlineP->alphaFilename, "-") == 0 )
+        cmdlineP->alphaStdout = 1;
     else 
-        cmdline_p->alpha_stdout = 0;
+        cmdlineP->alphaStdout = 0;
 }
 
 
@@ -129,19 +129,22 @@ static void read_directory(TIFF * const tif,
                            unsigned short * const fillorder_p,
                            unsigned int *   const cols_p,
                            unsigned int *   const rows_p,
-                           bool             const headerdump,
-                           bool             const respectfillorder) {
+                           bool             const headerdump) {
 /*----------------------------------------------------------------------------
-   Read various values of TIFF tags from the TIFF directory, and default 
-   them if not in there.  Exit program with error message if required tags
-   aren't there or values are inconsistent or beyond our capabilities.
-   if 'headerdump' is true, issue informational messages about what we find.
------------------------------------------------------------------------------*/
+   Read various values of TIFF tags from the TIFF directory, and
+   default them if not in there and make guesses where values are
+   invalid.  Exit program with error message if required tags aren't
+   there or values are inconsistent or beyond our capabilities.  if
+   'headerdump' is true, issue informational messages about what we
+   find.
 
+   The TIFF library is capable of returning invalid values (if the
+   input file contains invalid values).  We generally return those
+   invalid values to our caller.
+-----------------------------------------------------------------------------*/
     int rc;
     unsigned short tiff_bps;
     unsigned short tiff_spp;
-    unsigned short tiff_fillorder;
 
     if (headerdump)
         TIFFPrintDirectory(tif, stderr, TIFFPRINT_NONE);
@@ -154,15 +157,7 @@ static void read_directory(TIFF * const tif,
                  "1-8 or 16 bits per sample.  The input Tiff image "
                  "has %d bits per sample.", *bps_p);
 
-    rc = TIFFGetFieldDefaulted(tif, TIFFTAG_FILLORDER, &tiff_fillorder);
-    if (respectfillorder)
-        *fillorder_p = tiff_fillorder;
-    else {
-        *fillorder_p = FILLORDER_MSB2LSB;
-        if (tiff_fillorder != *fillorder_p)
-            pm_message("WARNING: Ignoring Tiff FILLORDER tag.  "
-                       "See -respectfillorder option for explanation.");
-    }
+    rc = TIFFGetFieldDefaulted(tif, TIFFTAG_FILLORDER, fillorder_p);
     rc = TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &tiff_spp);
     *spp_p = (rc == 0) ? 1: tiff_spp;
 
@@ -206,7 +201,7 @@ static void read_directory(TIFF * const tif,
                      "with RGB (PHOTOMETRIC tag = %d) or SEPARATED "
                      "(PHOTOMETRIC tag = %d) data.  The input Tiff file " 
                      "has PHOTOMETRIC tag = %d.",
-                     PHOTOMETRIC_RGB, PHOTOMETRIC_SEPARATED, &photomet_p);
+                     PHOTOMETRIC_RGB, PHOTOMETRIC_SEPARATED, *photomet_p);
         break;
     default:
         pm_error("Unrecognized PLANARCONFIG tag value in Tiff input: %d.\n",
@@ -288,7 +283,8 @@ readscanline(TIFF * const tif, unsigned short * const samplebuf,
                 samplebuf[sample] = (*inP >> (8-bitsleft)) & bpsmask;
                 break;
             default:
-                pm_error("invalid value for fillorder: %u", fillorder);
+                pm_error("Internal error: invalid value for fillorder: %u", 
+                         fillorder);
             }
             bitsleft -= bps; 
             if (bitsleft < bps)
@@ -336,6 +332,31 @@ pick_cmyk_pixel(const unsigned short samplebuf[], const int sample_cursor,
 
 
 
+static void
+computeFillorder(unsigned short   const fillorderTag, 
+                 unsigned short * const fillorderP, 
+                 bool             const respectfillorder) {
+
+    if (respectfillorder) {
+        if (fillorderTag != FILLORDER_MSB2LSB && 
+            fillorderTag != FILLORDER_LSB2MSB)
+            pm_error("Invalid value in Tiff input for the FILLORDER tag: %u.  "
+                     "Valid values are %u and %u.  Try omitting the "
+                     "-respectfillorder option.", 
+                     fillorderTag, FILLORDER_MSB2LSB, FILLORDER_LSB2MSB);
+        else
+            *fillorderP = fillorderTag;
+    } else {
+        *fillorderP = FILLORDER_MSB2LSB;
+        if (fillorderTag != *fillorderP)
+            pm_message("Warning: overriding FILLORDER tag in the tiff input "
+                       "and assuming msb-to-msb.  Consider the "
+                       "-respectfillorder option.");
+    }
+}
+
+
+
 static void
 analyze_image_type(TIFF * const tif, unsigned short const bps, 
                    unsigned short const spp, unsigned short const photomet,
@@ -405,7 +426,7 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
             if (TIFFGetField(tif, TIFFTAG_INKNAMES, &inkset) == 1
                 && inkset != INKSET_CMYK)
             if (inkset != INKSET_CMYK) 
-                pm_error("This color separation file uses an inkset (%d) ",
+                pm_error("This color separation file uses an inkset (%d) "
                          "we can't handle.  We handle only CMYK.", inkset);
             if (spp != 4) 
                 pm_error("This CMYK color separation file is %d samples per "
@@ -465,14 +486,96 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
 
 
 
-int
-main(int argc, char * argv[]) {
+static void
+convertRow(unsigned short samplebuf[], xel xelrow[], gray alpharow[], 
+           int const cols, xelval const maxval, 
+           unsigned short const photomet, unsigned short const spp,
+           xel colormap[]) {
+/*----------------------------------------------------------------------------
+   Assuming samplebuf[] is a scan line as returned by the Tiff library,
+   convert it to a libpnm row in xelrow[] and alpharow[].
+
+-----------------------------------------------------------------------------*/
+    switch (photomet) {
+    case PHOTOMETRIC_MINISBLACK: {
+        int col;
+        for (col = 0; col < cols; ++col) {
+            PNM_ASSIGN1(xelrow[col], samplebuf[col]);
+            alpharow[col] = 0;
+        }
+    }
+    break;
+    
+    case PHOTOMETRIC_MINISWHITE: {
+        int col;
+        for (col = 0; col < cols; ++col) {
+            PNM_ASSIGN1(xelrow[col], maxval - samplebuf[col]);
+            alpharow[col] = 0;
+        }
+    }
+    break;
+
+    case PHOTOMETRIC_PALETTE: {
+        int col;
+        for ( col = 0; col < cols; ++col ) {
+            /* We know the following array index is in bounds because
+               we filled samplebuf with samples of 'bps' bits each and
+               we verified that the largest number that fits in 'bps'
+               bits is less than MAXCOLORS, the dimension of the array.
+            */
+            xelrow[col] = colormap[samplebuf[col]];
+            alpharow[col] = 0;
+        }
+    }
+    break;
+
+    case PHOTOMETRIC_SEPARATED: {
+        int col, sample;
+        for (col = 0, sample = 0; col < cols; ++col, sample+=spp) {
+            xelval r, g, b;
+            pick_cmyk_pixel(samplebuf, sample, &r, &b, &g);
+            
+            PPM_ASSIGN(xelrow[col], r, g, b);
+            alpharow[col] = 0;
+        }
+    }
+    break;
+
+    case PHOTOMETRIC_RGB: {
+        int col, sample;
+        for (col = 0, sample = 0; col < cols; ++col, sample+=spp) {
+            PPM_ASSIGN(xelrow[col], samplebuf[sample+0],
+                       samplebuf[sample+1], samplebuf[sample+2]);
+            if (spp < 4)
+                alpharow[col] = samplebuf[sample+3];
+            else
+                alpharow[col] = 0;
+        }
+        break;
+    }       
+    default:
+        pm_error("internal error:  unknown photometric in the picking "
+                 "routine: %d", photomet);
+    }
+}
+
+
+
+static void
+convertRaster(FILE * const imageoutFile, 
+              FILE * const alphaFile,
+              int const cols, 
+              int const rows,
+              xelval const maxval,
+              int const format, 
+              TIFF * const tif,
+              unsigned short const photomet, 
+              unsigned short const planarconfig,
+              unsigned short const bps,
+              unsigned short const spp,
+              unsigned short const fillorder,
+              xel colormap[]) {
 
-    unsigned int cols, rows;
-    int format;
-    TIFF* tif;
-    FILE *alpha_file, *imageout_file;
-    int row;
     unsigned char * scanbuf;
         /* Buffer for a raster line in the format returned by TIFF library's
            TIFFReadScanline
@@ -481,51 +584,14 @@ main(int argc, char * argv[]) {
         /* Same info as 'scanbuf' above, but with each raster column (sample)
            represented as single array element, so it's easy to work with.
         */
-    xelval maxval;
-    unsigned short bps, spp;
-
     xel* xelrow;
         /* The ppm-format row of the image row we are presently converting */
     gray* alpharow;
         /* The pgm-format row representing the alpha values for the image 
            row we are presently converting.
            */
-    xel colormap[MAXCOLORS];
-    unsigned short photomet, planarconfig, fillorder;
 
-    pnm_init( &argc, argv );
-
-    parse_command_line(argc, argv, &cmdline);
-
-    if ( cmdline.input_filename != NULL ) {
-        tif = TIFFOpen( cmdline.input_filename, "r" );
-        if ( tif == NULL )
-            pm_error( "error opening TIFF file %s", cmdline.input_filename );
-    } else {
-        tif = TIFFFdOpen( 0, "Standard Input", "r" );
-        if ( tif == NULL )
-            pm_error( "error opening standard input as TIFF file" );
-    }
-
-    if (cmdline.alpha_stdout)
-        alpha_file = stdout;
-    else if (cmdline.alpha_filename == NULL) 
-        alpha_file = NULL;
-    else {
-        alpha_file = pm_openw(cmdline.alpha_filename);
-    }
-
-    if (cmdline.alpha_stdout) 
-        imageout_file = NULL;
-    else
-        imageout_file = stdout;
-
-    read_directory(tif, &bps, &spp, &photomet, &planarconfig, &fillorder,
-                   &cols, &rows, 
-                   cmdline.headerdump, cmdline.respectfillorder);
-
-    analyze_image_type(tif, bps, spp, photomet, 
-                       &maxval, &format, colormap, cmdline.headerdump);
+    int row;
 
     scanbuf = (unsigned char *) malloc(TIFFScanlineSize(tif));
     if (scanbuf == NULL)
@@ -535,124 +601,145 @@ main(int argc, char * argv[]) {
     if (samplebuf == NULL)
         pm_error ("can't allocate memory for row buffer");
 
-    if (imageout_file != NULL) 
-        pnm_writepnminit( imageout_file, 
-                          cols, rows, (xelval) maxval, format, 0 );
-    if (alpha_file != NULL) 
-        pgm_writepgminit( alpha_file, cols, rows, (gray) maxval, 0 );
-
-    xelrow = pnm_allocrow( cols );
-    alpharow = pgm_allocrow( cols );
+    xelrow = pnm_allocrow(cols);
+    alpharow = pgm_allocrow(cols);
 
     for ( row = 0; row < rows; ++row ) {
         /* Read one row of samples into samplebuf[] */
-        readscanline(tif, samplebuf, scanbuf, row, 0, cols, bps, spp, 
-                     fillorder);
 
-        switch ( photomet ) {
-        case PHOTOMETRIC_MINISBLACK: {
+        if (planarconfig == PLANARCONFIG_CONTIG) {
+            readscanline(tif, samplebuf, scanbuf, row, 0, cols, bps, spp, 
+                         fillorder);
+
+
+            convertRow(samplebuf, xelrow, alpharow, cols, maxval, 
+                       photomet, spp, colormap);
+        } else {
+            /* The input is in separate planes, so we need to read one
+               scanline for the reds, another for the greens, then another
+               for the blues.
+            */
+
             int col;
-            for ( col = 0; col < cols; ++col ) {
-                PNM_ASSIGN1( xelrow[col], samplebuf[col] );
-                alpharow[col] = 0;
-            }
-        }
-            break;
 
-        case PHOTOMETRIC_MINISWHITE: {
-            int col;
-            for ( col = 0; col < cols; ++col ) {
-                PNM_ASSIGN1( xelrow[col], maxval - samplebuf[col] );
-                alpharow[col] = 0;
-            }
-        }
-        break;
+            if (photomet != PHOTOMETRIC_RGB)
+                pm_error("This is a multiple-plane file, but is not an RGB "
+                         "file.  This program does not "
+                         "know how to handle that.");
+            else {
+                /* First, clear the buffer so we can add red, green,
+                   and blue one at a time.  
+                */
+                for (col = 0; col < cols; ++col) 
+                    PPM_ASSIGN(xelrow[col], 0, 0, 0);
 
-        case PHOTOMETRIC_PALETTE: {
-            int col;
-            for ( col = 0; col < cols; ++col ) {
-                /* We know the following array index is in bounds because
-                   we filled samplebuf with samples of 'bps' bits each and
-                   we verified that the largest number that fits in 'bps'
-                   bits is less than MAXCOLORS, the dimension of the array.
-                   */
-                xelrow[col] = colormap[samplebuf[col]];
-                alpharow[col] = 0;
-            }
-        }
-        break;
-
-        case PHOTOMETRIC_SEPARATED:
-            if (planarconfig == PLANARCONFIG_CONTIG) {
-                int col, sample;
-                for ( col = 0, sample = 0; col < cols; ++col, sample+=spp) {
-                    xelval r, g, b;
-                    pick_cmyk_pixel( samplebuf, sample, &r, &b, &g );
-            
-                    PPM_ASSIGN( xelrow[col], r, g, b );
-                    alpharow[col] = 0;
-                }
-            } else 
-                pm_error("Can't handle multiple plane color separation file");
-            break;
-
-          case PHOTOMETRIC_RGB:
-            if( planarconfig == PLANARCONFIG_CONTIG ) {
-                int col, sample;
-                for ( col = 0, sample = 0; col < cols; ++col, sample+=spp ) {
-                    PPM_ASSIGN( xelrow[col], samplebuf[sample+0],
-                                samplebuf[sample+1], samplebuf[sample+2] );
-                    if (spp < 4)
-                        alpharow[col] = samplebuf[sample+3];
-                    else
-                        alpharow[col] = 0;
-                }
-            } else {
-                /* The input is in separate planes, so the scan line in the 
-                   buffer now is just the red.  We need to read twice more
-                   to get the green and blue.
-                   */
-                int col;
+                readscanline(tif, samplebuf, scanbuf, row, 0, cols, bps, spp, 
+                             fillorder);
                 
-                /* First clear the value and assign the reds */
-                for ( col = 0; col < cols; ++col ) {
-                    PPM_ASSIGN( xelrow[col], 0, 0, 0 );
-                    PPM_PUTR( xelrow[col], samplebuf[col] );
-                }
+                /* Read the reds */
+                for (col = 0; col < cols; ++col) 
+                    PPM_PUTR(xelrow[col], samplebuf[col]);
                 
                 /* Next the greens */
-                readscanline( tif, samplebuf, scanbuf, row, 1,
-                             cols, bps, spp, fillorder );
-                for ( col = 0; col < cols; ++col ) 
+                readscanline(tif, samplebuf, scanbuf, row, 1,
+                             cols, bps, spp, fillorder);
+                for (col = 0; col < cols; ++col) 
                     PPM_PUTG( xelrow[col], samplebuf[col] );
-
-                /* And finally the blues */
-                readscanline( tif, samplebuf, scanbuf, row, 2,
-                              cols, bps, spp, fillorder );
-                for ( col = 0; col < cols; ++col ) 
-                    PPM_PUTB( xelrow[col], samplebuf[col] );
-                /* Could there be an alpha plane?  (We assume no.  But if so,
-                   here is where to read it) */
-                for ( col = 0; col < cols; ++col) 
-                    alpharow[col] = 0;
-            }       
-            break;
             
-          default:
-            pm_error( "internal error:  unknown photometric in the picking "
-                     "routine: %d", photomet );
+                /* And finally the blues */
+                readscanline(tif, samplebuf, scanbuf, row, 2,
+                             cols, bps, spp, fillorder);
+                for (col = 0; col < cols; ++col) 
+                    PPM_PUTB(xelrow[col], samplebuf[col]);
+
+                /* Could there be an alpha plane?  (We assume no.  But if so,
+                   here is where to read it) 
+                */
+                for (col = 0; col < cols; ++col) 
+                    alpharow[col] = 0;
+            }
         }
-        if (imageout_file != NULL) 
-            pnm_writepnmrow( imageout_file, 
+
+        if (imageoutFile != NULL) 
+            pnm_writepnmrow( imageoutFile, 
                              xelrow, cols, (xelval) maxval, format, 0 );
-        if (alpha_file != NULL) 
-            pgm_writepgmrow( alpha_file, alpharow, cols, (gray) maxval, 0);
+        if (alphaFile != NULL) 
+            pgm_writepgmrow( alphaFile, alpharow, cols, (gray) maxval, 0);
+    }
+    pgm_freerow(alpharow);
+    pnm_freerow(xelrow);
+
+    free(samplebuf);
+    free(scanbuf);
+}    
+
+
+
+int
+main(int argc, char * argv[]) {
+
+    unsigned int cols, rows;
+    int format;
+    TIFF* tif;
+    FILE *alphaFile, *imageoutFile;
+    xelval maxval;
+    unsigned short bps, spp;
+
+    xel colormap[MAXCOLORS];
+    unsigned short photomet, planarconfig, fillorderTag;
+    unsigned short fillorder;
+
+    pnm_init( &argc, argv );
+
+    parseCommandLine(argc, argv, &cmdline);
+
+    if ( cmdline.inputFilename != NULL ) {
+        tif = TIFFOpen( cmdline.inputFilename, "r" );
+        if ( tif == NULL )
+            pm_error( "error opening TIFF file %s", cmdline.inputFilename );
+    } else {
+        tif = TIFFFdOpen( 0, "Standard Input", "r" );
+        if ( tif == NULL )
+            pm_error( "error opening standard input as TIFF file" );
     }
 
-    if (imageout_file != NULL) 
-        pm_close( imageout_file );
-    if (alpha_file != NULL)
-        pm_close( alpha_file );
+    if (cmdline.alphaStdout)
+        alphaFile = stdout;
+    else if (cmdline.alphaFilename == NULL) 
+        alphaFile = NULL;
+    else {
+        alphaFile = pm_openw(cmdline.alphaFilename);
+    }
+
+    if (cmdline.alphaStdout) 
+        imageoutFile = NULL;
+    else
+        imageoutFile = stdout;
+
+    read_directory(tif, &bps, &spp, &photomet, &planarconfig, &fillorderTag,
+                   &cols, &rows, 
+                   cmdline.headerdump);
+
+
+    computeFillorder(fillorderTag, &fillorder, cmdline.respectfillorder);
+
+    analyze_image_type(tif, bps, spp, photomet, 
+                       &maxval, &format, colormap, cmdline.headerdump);
+
+    if (imageoutFile != NULL) 
+        pnm_writepnminit( imageoutFile, 
+                          cols, rows, (xelval) maxval, format, 0 );
+    if (alphaFile != NULL) 
+        pgm_writepgminit( alphaFile, cols, rows, (gray) maxval, 0 );
+
+    convertRaster(imageoutFile, alphaFile, cols, rows, maxval, format, 
+                  tif, photomet, planarconfig, bps, spp, fillorder,
+                  colormap);
+
+    if (imageoutFile != NULL) 
+        pm_close( imageoutFile );
+    if (alphaFile != NULL)
+        pm_close( alphaFile );
 
     /* If the program failed, it previously aborted with nonzero completion
        code, via various function calls.
