@@ -79,16 +79,8 @@
 #ifndef _BMP_H_
 #define _BMP_H_
 
-#include	"pbmplus.h"
+#include	"ppm.h"  /* For pm_error() */
 
-/* prototypes */
-static unsigned long BMPlenfileheader ARGS((int class));
-static unsigned long BMPleninfoheader ARGS((int class));
-static unsigned long BMPlencolormap ARGS((int class, unsigned long bitcount, int cmapsize));
-static unsigned long BMPlenline ARGS((int class, unsigned long bitcount, unsigned long x));
-static unsigned long BMPlenbits ARGS((int class, unsigned long bitcount, unsigned long x, unsigned long y));
-static unsigned long BMPlenfile ARGS((int class, unsigned long bitcount, int cmapsize, unsigned long x, unsigned long y));
-static unsigned long BMPoffbits ARGS((int class, unsigned long bitcount, int cmapsize));
 /*
  * Classes of BMP files
  */
