@@ -51,12 +51,12 @@
  * Initial revision
 \*/
 
-#include	"bmp.h"
-#include	"ppm.h"
-#include	"bitio.h"
+#include    "bmp.h"
+#include    "ppm.h"
+#include    "bitio.h"
 
 /* MAXCOLORS is the maximum size of a color map in a BMP image */
-#define	MAXCOLORS   	256
+#define MAXCOLORS       256
 /* BMPCOLORDEPTH is the number of intensities of a color channel (be it
    truecolor or in a color map) that are represented in a BMP image.
 */
@@ -79,7 +79,6 @@ static char    *ifname;
 static int GetByte ARGS((FILE * fp));
 static short GetShort ARGS((FILE * fp));
 static long GetLong ARGS((FILE * fp));
-static void readto ARGS((FILE *fp, unsigned long *ppos, unsigned long dst));
 static void BMPreadfileheader ARGS((FILE *fp, unsigned long *ppos,
     unsigned long *poffBits));
 static void BMPreadinfoheader ARGS((FILE *fp, unsigned long *ppos,
@@ -142,44 +141,44 @@ static char     er_read[] = "%s: read error";
 
 static int
 GetByte(fp)
-	FILE           *fp;
+    FILE           *fp;
 {
-	int             v;
+    int             v;
 
-	if ((v = getc(fp)) == EOF)
-	{
-		pm_error(er_read, ifname);
-	}
+    if ((v = getc(fp)) == EOF)
+    {
+        pm_error(er_read, ifname);
+    }
 
-	return v;
+    return v;
 }
 
 static short
 GetShort(fp)
-	FILE           *fp;
+    FILE           *fp;
 {
-	short           v;
+    short           v;
 
-	if (pm_readlittleshort(fp, &v) == -1)
-	{
-		pm_error(er_read, ifname);
-	}
+    if (pm_readlittleshort(fp, &v) == -1)
+    {
+        pm_error(er_read, ifname);
+    }
 
-	return v;
+    return v;
 }
 
 static long
 GetLong(fp)
-	FILE           *fp;
+    FILE           *fp;
 {
-	long            v;
+    long            v;
 
-	if (pm_readlittlelong(fp, &v) == -1)
-	{
-		pm_error(er_read, ifname);
-	}
+    if (pm_readlittlelong(fp, &v) == -1)
+    {
+        pm_error(er_read, ifname);
+    }
 
-	return v;
+    return v;
 }
 
 
@@ -189,25 +188,25 @@ readto(FILE *fp, unsigned long * const ppos, const unsigned long dst) {
    Read as many bytes as necessary to position the file 'fp' at offset 
    'dst', assuming we have already ready *ppos bytes from it.
 -----------------------------------------------------------------------------*/
-	unsigned long   pos;
+    unsigned long   pos;
 
-	if(!fp || !ppos)
-		return;
+    if(!fp || !ppos)
+        return;
 
-	pos = *ppos;
+    pos = *ppos;
 
-	if(pos > dst)
-		pm_error("%s: internal error in readto()", ifname);
+    if(pos > dst)
+        pm_error("%s: internal error in readto()", ifname);
 
-	for(; pos < dst; pos++)
-	{
-		if (getc(fp) == EOF)
-		{
-			pm_error(er_read, ifname);
-		}
-	}
+    for(; pos < dst; pos++)
+    {
+        if (getc(fp) == EOF)
+        {
+            pm_error(er_read, ifname);
+        }
+    }
 
-	*ppos = pos;
+    *ppos = pos;
 }
 
 /*
@@ -216,64 +215,64 @@ readto(FILE *fp, unsigned long * const ppos, const unsigned long dst) {
 
 static void
 BMPreadfileheader(fp, ppos, poffBits)
-	FILE           *fp;
-	unsigned long  *ppos;	/* number of bytes read from fp */
-	unsigned long  *poffBits;
+    FILE           *fp;
+    unsigned long  *ppos;   /* number of bytes read from fp */
+    unsigned long  *poffBits;
 {
-	unsigned long   cbSize;
-	unsigned short  xHotSpot;
-	unsigned short  yHotSpot;
-	unsigned long   offBits;
+    unsigned long   cbSize;
+    unsigned short  xHotSpot;
+    unsigned short  yHotSpot;
+    unsigned long   offBits;
 
-	if (GetByte(fp) != 'B')
-	{
-		pm_error("%s is not a BMP file", ifname);
-	}
-	if (GetByte(fp) != 'M')
-	{
-		pm_error("%s is not a BMP file", ifname);
-	}
+    if (GetByte(fp) != 'B')
+    {
+        pm_error("%s is not a BMP file", ifname);
+    }
+    if (GetByte(fp) != 'M')
+    {
+        pm_error("%s is not a BMP file", ifname);
+    }
 
-	cbSize = GetLong(fp);
-	xHotSpot = GetShort(fp);
-	yHotSpot = GetShort(fp);
-	offBits = GetLong(fp);
+    cbSize = GetLong(fp);
+    xHotSpot = GetShort(fp);
+    yHotSpot = GetShort(fp);
+    offBits = GetLong(fp);
 
-	*poffBits = offBits;
+    *poffBits = offBits;
 
-	*ppos += 14;
+    *ppos += 14;
 }
 
 static void
 BMPreadinfoheader(fp, ppos, pcx, pcy, pcBitCount, pclass, pcmapsize)
-	FILE           *fp;
-	unsigned long  *ppos;	/* number of bytes read from fp */
-	unsigned long  *pcx;
-	unsigned long  *pcy;
-	unsigned short *pcBitCount;
-	int            *pclass;
-	int	       *pcmapsize;
+    FILE           *fp;
+    unsigned long  *ppos;   /* number of bytes read from fp */
+    unsigned long  *pcx;
+    unsigned long  *pcy;
+    unsigned short *pcBitCount;
+    int            *pclass;
+    int        *pcmapsize;
 {
-	unsigned long   cbFix;
-	unsigned short  cPlanes;
+    unsigned long   cbFix;
+    unsigned short  cPlanes;
 
-	unsigned long   cx;
-	unsigned long   cy;
-	unsigned short  cBitCount;
+    unsigned long   cx;
+    unsigned long   cy;
+    unsigned short  cBitCount;
     unsigned long   compression;
-	int             class;
+    int             class;
 
-	cbFix = GetLong(fp);
+    cbFix = GetLong(fp);
 
-	switch (cbFix)
-	{
-	case 12:
-		class = C_OS2;
+    switch (cbFix)
+    {
+    case 12:
+        class = C_OS2;
 
-		cx = GetShort(fp);
-		cy = GetShort(fp);
-		cPlanes = GetShort(fp);
-		cBitCount = GetShort(fp);
+        cx = GetShort(fp);
+        cy = GetShort(fp);
+        cPlanes = GetShort(fp);
+        cBitCount = GetShort(fp);
         /* I actually don't know if the OS/2 BMP format allows
            cBitCount > 8 or if it does, what it means, but ppmtobmp
            creates such BMPs, more or less as a byproduct of creating
@@ -291,33 +290,33 @@ BMPreadinfoheader(fp, ppos, pcx, pcy, pcBitCount, pclass, pcmapsize)
             pm_error("Unrecognized bits per pixel in BMP file header: %d",
                      cBitCount);
 
-		break;
-	case 40: {
+        break;
+    case 40: {
         int colorsimportant;   /* ColorsImportant value from header */
         int colorsused;        /* ColorsUsed value from header */
 
-		class = C_WIN;
+        class = C_WIN;
 
-		cx = GetLong(fp);
-		cy = GetLong(fp);
-		cPlanes = GetShort(fp);
-		cBitCount = GetShort(fp);
+        cx = GetLong(fp);
+        cy = GetLong(fp);
+        cPlanes = GetShort(fp);
+        cBitCount = GetShort(fp);
  
-		compression = GetLong(fp);
+        compression = GetLong(fp);
         if (compression != 0) 
             pm_error("Input is compressed.  This program doesn't handle "
-                     "compressed images.  Compression type code = %d",
+                     "compressed images.  Compression type code = %ld",
                      compression);
 
         /* And read the rest of the junk in the 40 byte header */
-		GetLong(fp);   /* ImageSize */
-		GetLong(fp);   /* XpixelsPerM */
-		GetLong(fp);   /* YpixelsPerM */
-		colorsused = GetLong(fp);   /* ColorsUsed */
+        GetLong(fp);   /* ImageSize */
+        GetLong(fp);   /* XpixelsPerM */
+        GetLong(fp);   /* YpixelsPerM */
+        colorsused = GetLong(fp);   /* ColorsUsed */
         /* See comments in bmp.h for info about the definition of the following
            word and its relationship to the color map size (*pcmapsize).
            */
-		colorsimportant = GetLong(fp);  /* ColorsImportant */
+        colorsimportant = GetLong(fp);  /* ColorsImportant */
 
         if (colorsused != 0)
             *pcmapsize = colorsused;
@@ -334,49 +333,49 @@ BMPreadinfoheader(fp, ppos, pcx, pcy, pcBitCount, pclass, pcmapsize)
                          cBitCount);
         }
     }
-		break;
-	default:
-		pm_error("%s: unknown cbFix: %d", ifname, cbFix);
-		break;
-	}
+        break;
+    default:
+        pm_error("%s: unknown cbFix: %ld", ifname, cbFix);
+        break;
+    }
 
-	if (cPlanes != 1)
-	{
-		pm_error("%s: don't know how to handle cPlanes = %d"
-			 ,ifname
-			 ,cPlanes);
-	}
+    if (cPlanes != 1)
+    {
+        pm_error("%s: don't know how to handle cPlanes = %d"
+             ,ifname
+             ,cPlanes);
+    }
 
-	switch (class)
-	{
-	case C_WIN:
-		pm_message("Windows BMP, %dx%dx%d"
-			   ,cx
-			   ,cy
-			   ,cBitCount);
-		break;
-	case C_OS2:
-		pm_message("OS/2 BMP, %dx%dx%d"
-			   ,cx
-			   ,cy
-			   ,cBitCount);
-		break;
-	}
+    switch (class)
+    {
+    case C_WIN:
+        pm_message("Windows BMP, %ldx%ldx%d"
+               ,cx
+               ,cy
+               ,cBitCount);
+        break;
+    case C_OS2:
+        pm_message("OS/2 BMP, %ldx%ldx%d"
+               ,cx
+               ,cy
+               ,cBitCount);
+        break;
+    }
 
 #ifdef DEBUG
-	pm_message("cbFix: %d", cbFix);
-	pm_message("cx: %d", cx);
-	pm_message("cy: %d", cy);
-	pm_message("cPlanes: %d", cPlanes);
-	pm_message("cBitCount: %d", cBitCount);
+    pm_message("cbFix: %d", cbFix);
+    pm_message("cx: %d", cx);
+    pm_message("cy: %d", cy);
+    pm_message("cPlanes: %d", cPlanes);
+    pm_message("cBitCount: %d", cBitCount);
 #endif
 
-	*pcx = cx;
-	*pcy = cy;
-	*pcBitCount = cBitCount;
-	*pclass = class;
+    *pcx = cx;
+    *pcy = cy;
+    *pcBitCount = cBitCount;
+    *pclass = class;
 
-	*ppos += cbFix;
+    *ppos += cbFix;
 }
 
 /*
@@ -388,28 +387,28 @@ BMPreadcolormap(FILE *fp,
                 unsigned short cBitCount, 
                 int class, pixval *R, pixval *G, pixval *B, int cmapsize)
 {
-	int             i;
-	int		nbyte = 0;
+    int             i;
+    int     nbyte = 0;
 
-	for (i = 0; i < cmapsize; i++)
-	{
+    for (i = 0; i < cmapsize; i++)
+    {
         /* There is a document that says the bytes or ordered R,G,B,Z,
            but in practice it appears to be the following instead:
            */
-		B[i] = (pixval) GetByte(fp);
-		G[i] = (pixval) GetByte(fp);
-		R[i] = (pixval) GetByte(fp);
-		nbyte += 3;
+        B[i] = (pixval) GetByte(fp);
+        G[i] = (pixval) GetByte(fp);
+        R[i] = (pixval) GetByte(fp);
+        nbyte += 3;
 
-		if (class == C_WIN)
-		{
-			(void) GetByte(fp);
-			nbyte++;
-		}
-	}
+        if (class == C_WIN)
+        {
+            (void) GetByte(fp);
+            nbyte++;
+        }
+    }
 
-	*ppos += nbyte;
-	return nbyte;
+    *ppos += nbyte;
+    return nbyte;
 }
 
 static int
@@ -421,28 +420,28 @@ BMPreadrow(FILE *fp,
 /*
  * returns the number of bytes read, or -1 on error.
  */
-	BITSTREAM       b;
-	unsigned        nbyte = 0;
-	int             rc;
-	unsigned int    col;
+    BITSTREAM       b;
+    unsigned        nbyte = 0;
+    int             rc;
+    unsigned int    col;
 
-	if ((b = pm_bitinit(fp, "r")) == (BITSTREAM) 0)
-	{
-		return -1;
-	}
+    if ((b = pm_bitinit(fp, "r")) == (BITSTREAM) 0)
+    {
+        return -1;
+    }
 
-	for (col = 0; col < cx; col++)
-	{
-		unsigned long   v;
+    for (col = 0; col < cx; col++)
+    {
+        unsigned long   v;
 
         /* There is a document that gives a much different format for
            24 bit BMPs.  But this seems to be the de facto standard.
         */
-		if ((rc = pm_bitread(b, cBitCount, &v)) == -1)
-		{
-			return -1;
-		}
-		nbyte += rc;
+        if ((rc = pm_bitread(b, cBitCount, &v)) == -1)
+        {
+            return -1;
+        }
+        nbyte += rc;
 
         if (cBitCount <= 8)
             /* It's colormapped, so look up v in the colormap */
@@ -454,24 +453,24 @@ BMPreadrow(FILE *fp,
                        (v >>  8) & 0xff,
                        (v >> 16) & 0xff);
 
-	}
+    }
 
-	if ((rc = pm_bitfini(b)) != 0)
-	{
-		return -1;
-	}
+    if ((rc = pm_bitfini(b)) != 0)
+    {
+        return -1;
+    }
 
-	/*
-	 * Make sure we read a multiple of 4 bytes.
-	 */
-	while (nbyte % 4)
-	{
-		GetByte(fp);
-		nbyte++;
-	}
+    /*
+     * Make sure we read a multiple of 4 bytes.
+     */
+    while (nbyte % 4)
+    {
+        GetByte(fp);
+        nbyte++;
+    }
 
-	*ppos += nbyte;
-	return nbyte;
+    *ppos += nbyte;
+    return nbyte;
 }
 
 static pixel **
@@ -482,71 +481,71 @@ BMPreadbits(FILE *fp,
             unsigned short cBitCount, int class, 
             pixval *R, pixval *G, pixval *B)
 {
-	pixel         **pixels;	/* output */
-	long            y;
+    pixel         **pixels; /* output */
+    long            y;
 
-	readto(fp, ppos, offBits);
+    readto(fp, ppos, offBits);
 
-	pixels = ppm_allocarray(cx, cy);
+    pixels = ppm_allocarray(cx, cy);
 
-	/*
-	 * The picture is stored bottom line first, top line last
-	 */
+    /*
+     * The picture is stored bottom line first, top line last
+     */
 
-	for (y = cy - 1; y >= 0; y--)
-	{
-		int rc;
-		rc = BMPreadrow(fp, ppos, pixels[y], cx, cBitCount, R, G, B);
+    for (y = cy - 1; y >= 0; y--)
+    {
+        int rc;
+        rc = BMPreadrow(fp, ppos, pixels[y], cx, cBitCount, R, G, B);
 
-		if(rc == -1)
-		{
-			pm_error("%s: couldn't read row %d"
-				 ,ifname
-				 ,y);
-		}
-		if(rc%4)
-		{
-			pm_error("%s: row had bad number of bytes: %d"
-				 ,ifname
-				 ,rc);
-		}
-	}
+        if(rc == -1)
+        {
+            pm_error("%s: couldn't read row %ld"
+                     ,ifname
+                     ,y);
+        }
+        if(rc%4)
+        {
+            pm_error("%s: row had bad number of bytes: %d"
+                     ,ifname
+                     ,rc);
+        }
+    }
 
-	return pixels;
+    return pixels;
 }
 
 int
 main(argc, argv)
-	int             argc;
-	char          **argv;
+    int             argc;
+    char          **argv;
 {
     struct cmdline_info cmdline;
     FILE* ifp;
 
-	int             rc;
-	unsigned long	pos = 0;
+    int             rc;
+    unsigned long   pos = 0;
 
     /* The following are all information from the BMP headers */
-	unsigned long   offBits;
+    unsigned long   offBits;
       /* Byte offset into file of raster */
-	unsigned long   cx;
-	unsigned long   cy;
-	unsigned short  cBitCount;
-	int             class;
+    unsigned long   cx;
+    unsigned long   cy;
+    unsigned short  cBitCount;
+    int             class;
 
-	int		cmapsize;
+    int     cmapsize;
       /* Number of colors in the colormap in the BMP image */
 
     /* The colormap */
-	pixval          R[MAXCOLORS];	/* reds */
-	pixval          G[MAXCOLORS];	/* greens */
-	pixval          B[MAXCOLORS];	/* blues */
+    pixval          R[MAXCOLORS];   /* reds */
+    pixval          G[MAXCOLORS];   /* greens */
+    pixval          B[MAXCOLORS];   /* blues */
 
     /* The image */
-	pixel         **pixels;
+    pixel         **pixels;
 
 
-	ppm_init(&argc, argv);
+    ppm_init(&argc, argv);
 
     parse_command_line(argc, argv, &cmdline);
 
@@ -556,52 +555,52 @@ main(argc, argv)
     else 
         ifname = cmdline.input_filespec;
 
-	BMPreadfileheader(ifp, &pos, &offBits);
-	BMPreadinfoheader(ifp, &pos, &cx, &cy, &cBitCount, &class, &cmapsize);
+    BMPreadfileheader(ifp, &pos, &offBits);
+    BMPreadinfoheader(ifp, &pos, &cx, &cy, &cBitCount, &class, &cmapsize);
 
     if (cmdline.verbose) {
         pm_message("BMP image header says:");
         pm_message("  Class of BMP: %s", 
                    class == C_WIN ? "Windows" : class == C_OS2 ? "OS/2" :
                    "???");
-        pm_message("  Width: %d pixels", cx);
-        pm_message("  Height: %d pixels", cy);
-        pm_message("  Byte offset of raster within file: %d", offBits);
+        pm_message("  Width: %ld pixels", cx);
+        pm_message("  Height: %ld pixels", cy);
+        pm_message("  Byte offset of raster within file: %ld", offBits);
         pm_message("  Bits per pixel in raster: %d", cBitCount);
         pm_message("  Colors in color map: %d", cmapsize);
     }        
 
-	if(offBits != BMPoffbits(class, cBitCount, cmapsize))
-	{
-		pm_message("warning: offBits is %d, expected %d"
-			, offBits
-			, BMPoffbits(class, cBitCount, cmapsize));
-	}
+    if(offBits != BMPoffbits(class, cBitCount, cmapsize))
+    {
+        pm_message("warning: offBits is %ld, expected %ld"
+                   , offBits
+                   , BMPoffbits(class, cBitCount, cmapsize));
+    }
 
-	rc = BMPreadcolormap(ifp, &pos, cBitCount, class, R, G, B, cmapsize);
+    rc = BMPreadcolormap(ifp, &pos, cBitCount, class, R, G, B, cmapsize);
 
-	if(rc != BMPlencolormap(class, cBitCount, cmapsize))
-	{
-		pm_message("warning: %d-byte RGB table, expected %d bytes"
-			, rc
-			, BMPlencolormap(class, cBitCount, cmapsize));
-	}
+    if(rc != BMPlencolormap(class, cBitCount, cmapsize))
+    {
+        pm_message("warning: %d-byte RGB table, expected %ld bytes"
+                   , rc
+                   , BMPlencolormap(class, cBitCount, cmapsize));
+    }
 
 
-	pixels = BMPreadbits(ifp, &pos, offBits, cx, cy
-			, cBitCount, class, R, G, B);
+    pixels = BMPreadbits(ifp, &pos, offBits, cx, cy
+            , cBitCount, class, R, G, B);
 
-	if(pos != BMPlenfile(class, cBitCount, cmapsize, cx, cy))
-	{
-		pm_message("warning: read %d bytes, expected to read %d bytes"
-			, pos
-			, BMPlenfile(class, cBitCount, cmapsize, cx, cy));
-	}
+    if(pos != BMPlenfile(class, cBitCount, cmapsize, cx, cy))
+    {
+        pm_message("warning: read %ld bytes, expected to read %ld bytes"
+                   , pos
+                   , BMPlenfile(class, cBitCount, cmapsize, cx, cy));
+    }
 
-	pm_close(ifp);
-	ppm_writeppm(stdout, pixels, cx, cy, (pixval) BMPCOLORDEPTH-1, 0);
-	pm_close(stdout);
+    pm_close(ifp);
+    ppm_writeppm(stdout, pixels, cx, cy, (pixval) BMPCOLORDEPTH-1, 0);
+    pm_close(stdout);
 
-	exit(0);
+    exit(0);
 }
 
