@@ -31,10 +31,12 @@ ppm_computecolorhist( pixel ** const pixels,
     colorhist_vector chv;
 
     cht = ppm_computecolorhash(pixels, cols, rows, maxcolors, colorsP);
-    if (cht ==NULL)
-        return NULL;
-    chv = ppm_colorhashtocolorhist(cht, maxcolors);
-    ppm_freecolorhash(cht);
+    if (cht == NULL)
+        chv = NULL;
+    else {
+        chv = ppm_colorhashtocolorhist(cht, maxcolors);
+        ppm_freecolorhash(cht);
+    }
     return chv;
 }
 
@@ -94,7 +96,7 @@ ppm_addtocolorhist( colorhist_vector chv,
 
 
 
-colorhash_table
+static colorhash_table
 computecolorhash(pixel ** const pixels, 
                  const int cols, const int rows, 
                  const int maxcolors, int * const colorsP,
