@@ -13,37 +13,21 @@
 
 #include "ppm.h"
 
-#define NC 256			/* Total number of colors		*/
-#define NS 256			/* Max number of shades in primary	*/
-#define NS_WIDTH 8      /* Number of bits to hold a shade value */
-
 /* Besides having to have enough memory available, the limiting factor
    in the dithering matrix power is the size of the dithering value.
-   We need twice the dimension of the matrix plus space for the shade
-   value in an unsigned int.  
+   We need 2*dith_power bits in an unsigned int.  We also reserve
+   one bit to give headroom to do calculations with these numbers.
 */
-#define MAX_DITH_POWER ((sizeof(unsigned int)*8 - NS_WIDTH) / 2)
+#define MAX_DITH_POWER ((sizeof(unsigned int)*8 - 1) / 2)
 
 typedef unsigned char ubyte;
 
-static unsigned int dith_nr  =   5;	/* number of red shades			*/
-static unsigned int dith_ng  =   9;	/* number of green shades		*/
-static unsigned int dith_nb  =   5;	/* number of blue shades		*/
-static unsigned int dith_nc  = 225;	/* total number of colors 5 x 9 x 5	*/
 static unsigned int dith_power;     /* base 2 log of dither matrix dimension */
 static unsigned int dith_dim;      	/* dimension of the dither matrix	*/
 static unsigned int dith_dm2;      	/* dith_dim squared				*/
 static unsigned int **dith_mat; 	/* the dithering matrix			*/
 static int debug;
 
-/* prototypes */
-static unsigned int 
-dith_value (unsigned int y, unsigned int x, unsigned int dith_power);
-static void dith_matrix(void);
-static void dith_setup (unsigned int dith_power, 
-                        unsigned int nr, unsigned int ng, unsigned int nb, 
-                        pixel *ptab);
-
 /* COLOR():
  *	returns the index in the color table for the
  *      r, g, b values specified.
@@ -51,22 +35,25 @@ static void dith_setup (unsigned int dith_power,
 #define COLOR(r,g,b) (((r) * dith_ng + (g)) * dith_nb + (b))
 
 /* LEVELS():
- *	Returns the total number of levels after dithering.
+ * The number of shades we can represent by dithering of a primary whose
+ * individual pixels can express (s) shades of that primary.
  */
 #define LEVELS(s)     (dith_dm2 * ((s) - 1) + 1)
 
 /* DITHER():
  *	Returns the dithered color for a single primary.
- *      p = the input pixel
+ *      p = the primary value for the input pixel
+ *      m = the maxval on which 'p' is based
  *      d = entry in the dither matrix
- *      s = the number of levels of the primary
- *
+ *      s = the number of shades of this primary we can use in an 
+ *          individual output pixel
  */
-#define DITHER(p,d,s) ((ubyte) ((LEVELS(s) * (p) + (d)) / (dith_dm2 * NS)))
+#define DITHER(p,m,d,s) ((ubyte) ((LEVELS(s) * (p) / (m) + (d)) / dith_dm2))
+
 
 
 /* 
- *	Return the value of a dither matrix of which is 2**size elements
+ *	Return the value of a dither matrix which is 2**dith_power elements
  *  square at Row x, Column y.
  *	[graphics gems, p. 714]
  */
@@ -93,7 +80,6 @@ dith_value(unsigned int y, unsigned int x, const unsigned int dith_power) {
 
 /*
  *	Form the dithering matrix for the dimension specified
- *	(Scaled by NS)
  *
  *  Note that we assume 'dith_dim' is small enough that the dith_mat_sz
  *  computed within fits in an int.  Otherwise, results are undefined.
@@ -120,7 +106,7 @@ dith_matrix(void)
 
     for (y = 0; y < dith_dim; y++) {
         for (x = 0; x < dith_dim; x++) {
-            dith_mat[y][x] = NS * dith_value(y, x, dith_power);
+            dith_mat[y][x] = dith_value(y, x, dith_power);
             if (debug)
                 (void) fprintf(stderr, "%4d ", dith_mat[y][x]);
         }
@@ -132,39 +118,40 @@ dith_matrix(void)
     
 /*        
  *	Setup the dithering parameters, lookup table and dithering matrix
+ *
+ *  Malloc the lookup table and return its address as *ptab.
+
  */
-void
+static void
 dith_setup(const unsigned int dith_power, 
-           const unsigned int nr, 
-           const unsigned int ng, 
-           const unsigned int nb, 
-           pixel ptab[])
+           const unsigned int dith_nr, 
+           const unsigned int dith_ng, 
+           const unsigned int dith_nb, 
+           const pixval output_maxval,
+           pixel ** const ptab_p)
 {
-    unsigned int r, g, b, i;
+    unsigned int r, g, b;
 
-    dith_nr  = nr;
-    dith_ng  = ng;
-    dith_nb  = nb;
-    dith_nc  = nr * ng * nb;
-
-    if (dith_nc > NC)
-	pm_error("too many shades %d, max %d", dith_nc, NC);
     if (dith_nr < 2) 
 	pm_error("too few shades for red, minimum of 2");
     if (dith_ng < 2) 
 	pm_error("too few shades for green, minimum of 2");
     if (dith_nb < 2) 
 	pm_error("too few shades for blue, minimum of 2");
+
+    *ptab_p = malloc(dith_nr * dith_ng * dith_nb * sizeof(pixel));
+    if (*ptab_p == NULL) 
+        pm_error("Unable to allocate space for the color lookup table "
+                 "(%d by %d by %d pixels).", dith_nr, dith_ng, dith_nb);
     
     for (r = 0; r < dith_nr; r++) 
-	for (g = 0; g < dith_ng; g++) 
-	    for (b = 0; b < dith_nb; b++) {
-		i = COLOR(r,g,b);
-		PPM_ASSIGN(ptab[COLOR(r,g,b)], 
-		           (r * (NC-1) / (dith_nr - 1)),
-		           (g * (NC-1) / (dith_ng - 1)),
-		           (b * (NC-1) / (dith_nb - 1)));
-	    }
+        for (g = 0; g < dith_ng; g++) 
+            for (b = 0; b < dith_nb; b++) {
+                PPM_ASSIGN((*ptab_p)[COLOR(r,g,b)], 
+                           (r * output_maxval / (dith_nr - 1)),
+                           (g * output_maxval / (dith_ng - 1)),
+                           (b * output_maxval / (dith_nb - 1)));
+            }
     
     if (dith_power > MAX_DITH_POWER) {
         pm_error("Dithering matrix power %d is too large.  Must be <= %d",
@@ -183,8 +170,14 @@ dith_setup(const unsigned int dith_power,
  */
 static void
 dith_dither(const unsigned int width, const unsigned int height, 
+            const pixval maxval,
             const pixel * const ptab, 
-            pixel ** const input, pixel ** const output) {
+            pixel ** const input, pixel ** const output,
+            const unsigned int dith_nr,
+            const unsigned int dith_ng,
+            const unsigned int dith_nb, 
+            const pixval output_maxval
+            ) {
 
     const unsigned int dm = (dith_dim - 1);  /* A mask */
     unsigned int row, col; 
@@ -194,9 +187,9 @@ dith_dither(const unsigned int width, const unsigned int height,
             const unsigned int d = dith_mat[row & dm][(width-col-1) & dm];
             const pixel input_pixel = input[row][col];
             output[row][col] = 
-                ptab[COLOR(DITHER(PPM_GETR(input_pixel), d, dith_nr), 
-                           DITHER(PPM_GETG(input_pixel), d, dith_ng), 
-                           DITHER(PPM_GETB(input_pixel), d, dith_nb))];
+                ptab[COLOR(DITHER(PPM_GETR(input_pixel), maxval, d, dith_nr), 
+                           DITHER(PPM_GETG(input_pixel), maxval, d, dith_ng), 
+                           DITHER(PPM_GETB(input_pixel), maxval, d, dith_nb))];
 
         }
 }
@@ -208,18 +201,26 @@ main( argc, argv )
     char* argv[];
     {
     FILE* ifp;
-    pixel ptab[256];
+    pixel *ptab;    /* malloc'd */
     pixel **ipixels;        /* Input image */
     pixel **opixels;        /* Output image */
     int cols, rows;
-    pixval maxval;
+    pixval maxval;  /* Maxval of the input image */
+    pixval output_maxval;  /* Maxval in the dithered output image */
     unsigned int argn;
     char* usage = 
 	"[-dim <num>] [-red <num>] [-green <num>] [-blue <num>] [ppmfile]";
+    unsigned int dith_nr; /* number of red shades in output */
+    unsigned int dith_ng; /* number of green shades	in output */
+    unsigned int dith_nb; /* number of blue shades in output */
 
 
     ppm_init( &argc, argv );
 
+    dith_nr = 5;  /* default */
+    dith_ng = 9;  /* default */
+    dith_nb = 5;  /* default */
+
     dith_power = 4;  /* default */
     debug = 0; /* default */
     argn = 1;
@@ -268,10 +269,11 @@ main( argc, argv )
     ipixels = ppm_readppm( ifp, &cols, &rows, &maxval );
     pm_close( ifp );
     opixels = ppm_allocarray(cols, rows);
-    maxval = 255;
-    dith_setup(dith_power, dith_nr, dith_ng, dith_nb, ptab);
-    dith_dither(cols, rows, ptab, ipixels, opixels);
-    ppm_writeppm(stdout, opixels, cols, rows, maxval, 0);
+    output_maxval = pm_lcm(dith_nr-1, dith_ng-1, dith_nb-1, PPM_MAXMAXVAL);
+    dith_setup(dith_power, dith_nr, dith_ng, dith_nb, output_maxval, &ptab);
+    dith_dither(cols, rows, maxval, ptab, ipixels, opixels,
+                dith_nr, dith_ng, dith_nb, output_maxval);
+    ppm_writeppm(stdout, opixels, cols, rows, output_maxval, 0);
     pm_close(stdout);
     exit(0);
 }
