@@ -5,6 +5,16 @@
 **
 ** Modified by Marcel Wijkstra <wijkstra@fwi.uva.nl>
 **
+** The non-LZW GIF generation stuff was adapted from the Independent
+** JPEG Group's djpeg on 2001.09.29.  The uncompressed output subroutines
+** are derived directly from the corresponding subroutines in djpeg's
+** wrgif.c source file.  Its copyright notice say:
+
+ * Copyright (C) 1991-1997, Thomas G. Lane.
+ * This file is part of the Independent JPEG Group's software.
+ * For conditions of distribution and use, see the accompanying README file.
+ *
+   The reference README file is README.JPEG in the Netpbm package.
 **
 ** Copyright (C) 1989 by Jef Poskanzer.
 **
@@ -20,6 +30,10 @@
 ** CompuServe Incorporated.
 */
 
+/* TODO: merge the LZW and uncompressed subroutines.  They are separate
+   only because they had two different lineages and the code is too
+   complicated for me quickly to rewrite it.
+*/
 #include "ppm.h"
 #include "ppmcmap.h"
 
@@ -66,15 +80,20 @@ struct cmap {
         /* The maxval for the colors in 'cht'. */
 };
 
-static void BumpPixel ARGS(( void ));
-static void Putword ARGS(( int w, FILE* fp ));
-static void output ARGS(( code_int code ));
-static void cl_block ARGS(( void ));
-static void cl_hash ARGS(( count_int hsize ));
-static void writeerr ARGS(( void ));
-static void char_init ARGS(( void ));
-static void char_out ARGS(( int c ));
-static void flush_char ARGS(( void ));
+struct gif_dest {
+    /* This structure controls output of uncompressed GIF raster */
+
+    /* State for packing variable-width codes into a bitstream */
+    int n_bits;         /* current number of bits/code */
+    int maxcode;        /* maximum code, given n_bits */
+    int cur_accum;      /* holds bits not yet output */
+    int cur_bits;       /* # of bits in cur_accum */
+
+    /* State for GIF code assignment */
+    int ClearCode;      /* clear code (doesn't change) */
+    int EOFCode;        /* EOF code (ditto) */
+    int code_counter;   /* counts output symbols */
+};
 
 struct cmdline_info {
     /* All the information the user supplied in the command line,
@@ -88,35 +107,77 @@ struct cmdline_info {
     char *mapfile;         /* -mapfile option value.  NULL if none. */
     char *transparent;     /* -transparent option value.  NULL if none. */
     char *comment;         /* -comment option value; NULL if none */
+    unsigned int nolzw;    /* -nolzw option */
 };
 
 
+static void
+handleLatex2htmlHack(void) {
+/*----------------------------------------------------------------------------
+  This program used to put out a "usage" message when it saw an option
+  it didn't understand.  Latex2html's configure program does a
+  ppmtogif -h (-h was never a valid option) to elicit that message and
+  then parses the message to see if it included the strings
+  "-interlace" and "-transparent".  That way it knows if the
+  'ppmtogif' program it found has those options or not.  I don't think
+  any 'ppmtogif' you're likely to find today lacks those options, but
+  latex2html checks anyway, and we don't want it to conclude that we
+  don't have them.
+
+  So we issue a special error message just to trick latex2html into
+  deciding that we have -interlace and -transparent options.  The function
+  is not documented in the man page.  We would like to see Latex2html 
+  either stop checking or check like configure programs usually do -- 
+  try the option and see if you get success or failure.
+
+  -Bryan 2001.11.14
+-----------------------------------------------------------------------------*/
+     pm_error("latex2html, you should just try the -interlace and "
+             "-transparent options to see if they work instead of "
+             "expecting a 'usage' message from -h");
+}
+
+
+
 static void
 parse_command_line(int argc, char ** argv,
                    struct cmdline_info *cmdline_p) {
 /*----------------------------------------------------------------------------
+   Parse the program arguments (given by argc and argv) into a form
+   the program can deal with more easily -- a cmdline_info structure.
+   If the syntax is invalid, issue a message and exit the program via
+   pm_error().
+
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
-         */
-    optStruct2 opt;
-
+    optStruct3 opt;  /* set by OPTENT3 */
+    optEntry *option_def = malloc(100*sizeof(optEntry));
     unsigned int option_def_index;
 
-    option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0,   "interlace",   OPT_FLAG,   &cmdline_p->interlace,      0);
-    OPTENTRY(0,   "sort",        OPT_FLAG,   &cmdline_p->sort,           0);
-    OPTENTRY(0,   "mapfile",     OPT_STRING, &cmdline_p->mapfile,        0);
-    OPTENTRY(0,   "transparent", OPT_STRING, &cmdline_p->transparent,    0);
-    OPTENTRY(0,   "comment",     OPT_STRING, &cmdline_p->comment,        0);
-    OPTENTRY(0,   "alpha",       OPT_STRING, &cmdline_p->alpha_filespec, 0);
-    OPTENTRY(0,   "alphacolor",  OPT_STRING, &cmdline_p->alphacolor,     0);
+    unsigned int latex2htmlhack;
 
+    option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0,   "interlace",   OPT_FLAG,   
+            NULL,                       &cmdline_p->interlace, 0);
+    OPTENT3(0,   "sort",        OPT_FLAG,   
+            NULL,                       &cmdline_p->sort, 0);
+    OPTENT3(0,   "nolzw",       OPT_FLAG,   
+            NULL,                       &cmdline_p->nolzw, 0);
+    OPTENT3(0,   "mapfile",     OPT_STRING, 
+            &cmdline_p->mapfile,        NULL, 0);
+    OPTENT3(0,   "transparent", OPT_STRING, 
+            &cmdline_p->transparent,    NULL, 0);
+    OPTENT3(0,   "comment",     OPT_STRING, 
+            &cmdline_p->comment,        NULL, 0);
+    OPTENT3(0,   "alpha",       OPT_STRING, 
+            &cmdline_p->alpha_filespec, NULL, 0);
+    OPTENT3(0,   "alphacolor",  OPT_STRING, 
+            &cmdline_p->alphacolor,     NULL, 0);
+    OPTENT3(0,   "h",           OPT_FLAG, 
+            NULL,                       &latex2htmlhack, 0);
+    
     /* Set the defaults */
-    cmdline_p->interlace = FALSE;
-    cmdline_p->sort = FALSE;
     cmdline_p->mapfile = NULL;
     cmdline_p->transparent = NULL;  /* no transparency */
     cmdline_p->comment = NULL;      /* no comment */
@@ -128,11 +189,14 @@ parse_command_line(int argc, char ** argv,
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
-    opt.allowNegNum = FALSE;  /* We may have parms that are negative numbers */
+    opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    pm_optParseOptions2(&argc, argv, opt, 0);
+    pm_optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
+    if (latex2htmlhack) 
+        handleLatex2htmlHack();
+
     if (argc-1 == 0) 
         cmdline_p->input_filespec = "-";
     else if (argc-1 != 1)
@@ -152,6 +216,17 @@ static int __inline__ sqr(const int x) {
 }
 
 
+/*
+ * Write out a word to the GIF file
+ */
+static void
+Putword(int const w, FILE * const fp) {
+
+    fputc( w & 0xff, fp );
+    fputc( (w / 256) & 0xff, fp );
+}
+
+
 static int __inline__
 closestcolor(const pixel color, pixval const maxval, struct cmap * cmapP) {
 /*----------------------------------------------------------------------------
@@ -457,135 +532,82 @@ static FILE* g_outfile;
 static int ClearCode;
 static int EOFCode;
 
+/***************************************************************************
+*                          BYTE OUTPUTTER                 
+***************************************************************************/
+
+static char accum[256];
+    /* The current packet, under construction */
+static int a_count;
+    /* Number of characters so far in the current packet */
+
 static void
-compress(pixel ** const pixels, pixval const input_maxval,
-         gray ** const alpha, gray const alpha_maxval, 
-         struct cmap * const cmapP, 
-         int const init_bits, FILE * const outfile) {
-
-    long fcode;
-    code_int i /* = 0 */;
-    int c;
-    code_int ent;
-    code_int disp;
-    code_int hsize_reg;
-    int hshift;
-    bool eof;
-    gray const alpha_threshold = (alpha_maxval + 1) / 2;
-        /* gray levels below this in the alpha mask indicate transparent
-           pixels in the output image.
-        */
-    
-    /*
-     * Set up the globals:  g_init_bits - initial number of bits
-     *                      g_outfile   - pointer to output file
-     */
-    g_init_bits = init_bits;
-    g_outfile = outfile;
-
-    /*
-     * Set up the necessary values
-     */
-    offset = 0;
-    out_count = 0;
-    clear_flg = 0;
-    in_count = 1;
-    maxcode = MAXCODE(n_bits = g_init_bits);
-
-    ClearCode = (1 << (init_bits - 1));
-    EOFCode = ClearCode + 1;
-    free_ent = ClearCode + 2;
-
-    char_init();
-
-    ent = GIFNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
-
-    hshift = 0;
-    for ( fcode = (long) hsize;  fcode < 65536L; fcode *= 2L )
-        ++hshift;
-    hshift = 8 - hshift;                /* set hash code range bound */
-
-    hsize_reg = hsize;
-    cl_hash( (count_int) hsize_reg);            /* clear hash table */
-
-    output( (code_int)ClearCode );
-
-    eof = FALSE;
-    while (!eof) {
-        c = GIFNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
-        if (c == EOF) eof = TRUE;
-        if (!eof) {
-            ++in_count;
-
-            fcode = (long) (((long) c << maxbits) + ent);
-            i = (((code_int)c << hshift) ^ ent);    /* xor hashing */
-
-            if ( HashTabOf (i) == fcode ) {
-                ent = CodeTabOf (i);
-                continue;
-            } else if ( (long)HashTabOf (i) < 0 )      /* empty slot */
-                goto nomatch;
-            disp = hsize_reg - i;        /* secondary hash (after G. Knott) */
-            if ( i == 0 )
-                disp = 1;
-        probe:
-            if ( (i -= disp) < 0 )
-                i += hsize_reg;
-
-            if ( HashTabOf (i) == fcode ) {
-                ent = CodeTabOf (i);
-                continue;
-            }
-            if ( (long)HashTabOf (i) > 0 )
-                goto probe;
-        nomatch:
-            output ( (code_int) ent );
-            ++out_count;
-            ent = c;
-            if ( free_ent < maxmaxcode ) {  /* } */
-                CodeTabOf (i) = free_ent++; /* code -> hashtable */
-                HashTabOf (i) = fcode;
-            } else
-                cl_block();
-        }
-    }
-    /*
-     * Put out the final code.
-     */
-    output( (code_int)ent );
-    ++out_count;
-    output( (code_int) EOFCode );
+char_init() {
+    a_count = 0;
 }
 
-/*****************************************************************
- * TAG( output )
- *
- * Output the given code.
+
+
+static void
+flush_char() {
+/*----------------------------------------------------------------------------
+   Write the current packet to the output file, then reset the current 
+   packet to empty.
+-----------------------------------------------------------------------------*/
+    if( a_count > 0 ) {
+        fputc( a_count, g_outfile );
+        fwrite( accum, 1, a_count, g_outfile );
+        a_count = 0;
+    }
+}
+
+
+
+static void
+char_out( int const c ) {
+/*----------------------------------------------------------------------------
+   Add a character to the end of the current packet, and if it is 254
+   character, flush the packet to the output file.
+-----------------------------------------------------------------------------*/
+    accum[ a_count++ ] = c;
+    if( a_count >= 254 )
+        flush_char();
+}
+
+
+
+static unsigned long const masks[] = { 0x0000, 0x0001, 0x0003, 0x0007, 0x000F,
+                                       0x001F, 0x003F, 0x007F, 0x00FF,
+                                       0x01FF, 0x03FF, 0x07FF, 0x0FFF,
+                                       0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF };
+
+static void
+output( code_int const code, FILE * const outfile ) {
+/*----------------------------------------------------------------------------
+   Output one GIF code to the file.
+
+   The code is represented as cur_bits+n_bits/8 bytes in the file.
+
  * Inputs:
  *      code:   A n_bits-bit integer.  If == -1, then EOF.  This assumes
  *              that n_bits =< (long)wordsize - 1.
- * Outputs:
- *      Outputs code to the file.
  * Assumptions:
  *      Chars are 8 bits long.
  * Algorithm:
  *      Maintain a BITS character long buffer (so that 8 codes will
  * fit in it exactly).  Use the VAX insv instruction to insert each
  * code in turn.  When the buffer fills up empty it and start over.
- */
 
-static unsigned long cur_accum = 0;
-static int cur_bits = 0;
+-----------------------------------------------------------------------------*/
+    /* Note: this initialization of static variables by the loader must go
+       if we want to use the compressor more than once per invocation
+       (e.g. we want to create a multi-image gif).
+    */
+    static unsigned long cur_accum = 0;
+    static int cur_bits = 0;
 
-static unsigned long masks[] = { 0x0000, 0x0001, 0x0003, 0x0007, 0x000F,
-                                  0x001F, 0x003F, 0x007F, 0x00FF,
-                                  0x01FF, 0x03FF, 0x07FF, 0x0FFF,
-                                  0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF };
+    g_outfile = outfile;
 
-static void
-output( code )
-code_int  code;
-{
     cur_accum &= masks[ cur_bits ];
 
     if( cur_bits > 0 )
@@ -629,7 +651,9 @@ code_int  code;
         while( cur_bits > 0 ) {
                 char_out( (unsigned int)(cur_accum & 0xff) );
                 cur_accum >>= 8;
-                cur_bits -= 8;
+                cur_bits -= 8;  
+                    /* ?? what if cur_bits < 8?  Apparently only an issue 
+                       if we extend this program to do multiple images */
         }
 
         flush_char();
@@ -637,23 +661,11 @@ code_int  code;
         fflush( g_outfile );
 
         if( ferror( g_outfile ) )
-                writeerr();
+            pm_error( "error writing output file" );
     }
 }
 
-/*
- * Clear out the hash table
- */
-static void
-cl_block ()             /* table clear for block compress */
-{
 
-        cl_hash ( (count_int) hsize );
-        free_ent = ClearCode + 2;
-        clear_flg = 1;
-
-        output( (code_int)ClearCode );
-}
 
 static void
 cl_hash(hsize)          /* reset code table */
@@ -690,91 +702,278 @@ register count_int hsize;
                 *--htab_p = m1;
 }
 
+
+/*
+ * Clear out the hash table
+ */
 static void
-writeerr()
+cl_block (FILE * const outfile)     /* table clear for block compress */
 {
-        pm_error( "error writing output file" );
+
+        cl_hash ( (count_int) hsize );
+        free_ent = ClearCode + 2;
+        clear_flg = 1;
+
+        output( (code_int)ClearCode, outfile );
 }
 
+
+
+static void
+write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
+                 gray ** const alpha, gray const alpha_maxval, 
+                 struct cmap * const cmapP, 
+                 int const init_bits, FILE * const outfile) {
+/*----------------------------------------------------------------------------
+   Write the raster to file 'outfile'.
+
+   The raster to write is 'pixels', which has maxval 'input_maxval',
+   modified by alpha mask 'alpha', which has maxval 'alpha_maxval'.
+
+   Use the colormap 'cmapP' to generate the raster ('pixels' is 
+   composed of RGB samples; the GIF raster is colormap indices).
+
+   Write the raster using LZW compression.
+-----------------------------------------------------------------------------*/
+
+    code_int ent;
+    code_int disp;
+    code_int hsize_reg;
+    int hshift;
+    bool eof;
+    gray const alpha_threshold = (alpha_maxval + 1) / 2;
+        /* gray levels below this in the alpha mask indicate transparent
+           pixels in the output image.
+        */
+    
+    /*
+     * Set up the globals:  g_init_bits - initial number of bits
+     *                      g_outfile   - pointer to output file
+     */
+    g_init_bits = init_bits;
+
+    /*
+     * Set up the necessary values
+     */
+    offset = 0;
+    out_count = 0;
+    clear_flg = 0;
+    in_count = 1;
+    maxcode = MAXCODE(n_bits = g_init_bits);
+
+    ClearCode = (1 << (init_bits - 1));
+    EOFCode = ClearCode + 1;
+    free_ent = ClearCode + 2;
+
+    char_init();
+
+    ent = GIFNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
+
+    {
+        long fcode;
+        hshift = 0;
+        for ( fcode = (long) hsize;  fcode < 65536L; fcode *= 2L )
+            ++hshift;
+        hshift = 8 - hshift;                /* set hash code range bound */
+    }
+    hsize_reg = hsize;
+    cl_hash( (count_int) hsize_reg);            /* clear hash table */
+
+    output( (code_int)ClearCode, outfile );
+
+    eof = FALSE;
+    while (!eof) {
+        int gifpixel;
+            /* The value for the pixel in the GIF image.  I.e. the colormap
+               index.  Or -1 to indicate "no more pixels."
+            */
+        gifpixel = GIFNextPixel(pixels, 
+                                input_maxval, alpha, alpha_threshold, cmapP);
+        if (gifpixel == EOF) eof = TRUE;
+        if (!eof) {
+            long const fcode = (long) (((long) gifpixel << maxbits) + ent);
+            code_int i;
+                /* xor hashing */
+
+            ++in_count;
+
+            i = (((code_int)gifpixel << hshift) ^ ent);    
+
+            if ( HashTabOf (i) == fcode ) {
+                ent = CodeTabOf (i);
+                continue;
+            } else if ( (long)HashTabOf (i) < 0 )      /* empty slot */
+                goto nomatch;
+            disp = hsize_reg - i;        /* secondary hash (after G. Knott) */
+            if ( i == 0 )
+                disp = 1;
+        probe:
+            if ( (i -= disp) < 0 )
+                i += hsize_reg;
+
+            if ( HashTabOf (i) == fcode ) {
+                ent = CodeTabOf (i);
+                continue;
+            }
+            if ( (long)HashTabOf (i) > 0 )
+                goto probe;
+        nomatch:
+            output ( (code_int) ent, outfile );
+            ++out_count;
+            ent = gifpixel;
+            if ( free_ent < maxmaxcode ) {  /* } */
+                CodeTabOf (i) = free_ent++; /* code -> hashtable */
+                HashTabOf (i) = fcode;
+            } else
+                cl_block(outfile);
+        }
+    }
+    /*
+     * Put out the final code.
+     */
+    output( (code_int)ent, outfile );
+    ++out_count;
+    output( (code_int) EOFCode, outfile );
+}
+
+
+
+/* Routine to convert variable-width codes into a byte stream */
+
+static void
+output_uncompressed(struct gif_dest * const dinfo, int const code,
+                    FILE * const outfile) {
+
+    g_outfile = outfile;
+
+    /* Emit a code of n_bits bits */
+    /* Uses cur_accum and cur_bits to reblock into 8-bit bytes */
+    dinfo->cur_accum |= ((int) code) << dinfo->cur_bits;
+    dinfo->cur_bits += dinfo->n_bits;
+
+    while (dinfo->cur_bits >= 8) {
+        char_out(dinfo->cur_accum & 0xFF);
+        dinfo->cur_accum >>= 8;
+        dinfo->cur_bits -= 8;
+    }
+}
+
+
+static void
+write_raster_uncompressed_init (struct gif_dest * const dinfo, 
+                                int const i_bits, FILE * const outfile)
+/* Initialize pseudo-compressor */
+{
+  /* init all the state variables */
+  dinfo->n_bits = i_bits;
+  dinfo->maxcode = MAXCODE(dinfo->n_bits);
+  dinfo->ClearCode = (1 << (i_bits - 1));
+  dinfo->EOFCode = dinfo->ClearCode + 1;
+  dinfo->code_counter = dinfo->ClearCode + 2;
+  /* init output buffering vars */
+  dinfo->cur_accum = 0;
+  dinfo->cur_bits = 0;
+  /* GIF specifies an initial Clear code */
+  output_uncompressed(dinfo, dinfo->ClearCode, outfile);
+}
+
+
+static void
+write_raster_uncompressed_pixel (struct gif_dest * const dinfo, 
+                                 int const c, FILE * const outfile)
+/* Accept and "compress" one pixel value.
+ * The given value must be less than n_bits wide.
+ */
+{
+  /* Output the given pixel value as a symbol. */
+  output_uncompressed(dinfo, c, outfile);
+  /* Issue Clear codes often enough to keep the reader from ratcheting up
+   * its symbol size.
+   */
+  if (dinfo->code_counter < dinfo->maxcode) {
+    dinfo->code_counter++;
+  } else {
+    output_uncompressed(dinfo, dinfo->ClearCode, outfile);
+    dinfo->code_counter = dinfo->ClearCode + 2;	/* reset the counter */
+  }
+}
+
+
+static void
+write_raster_uncompressed_term (struct gif_dest * const dinfo,
+                                FILE * const outfile)
+/* Clean up at end */
+{
+  /* Send an EOF code */
+  output_uncompressed(dinfo, dinfo->EOFCode, outfile);
+  /* Flush the bit-packing buffer */
+  if (dinfo->cur_bits > 0) {
+    char_out(dinfo->cur_accum & 0xFF);
+  }
+  /* Flush the packet buffer */
+  flush_char();
+}
+
+
+
+static void
+write_raster_uncompressed(pixel ** const pixels, pixval const input_maxval,
+                          gray ** const alpha, gray const alpha_maxval, 
+                          struct cmap * const cmapP, 
+                          int const init_bits, FILE * const outfile) {
+/*----------------------------------------------------------------------------
+   Write the raster to file 'outfile'.
+   
+   Same as write_raster_LZW(), except written out one code per
+   pixel (plus some clear codes), so no compression.  And no use
+   of the LZW patent.
+-----------------------------------------------------------------------------*/
+    gray const alpha_threshold = (alpha_maxval + 1) / 2;
+        /* gray levels below this in the alpha mask indicate transparent
+           pixels in the output image.
+        */
+    bool eof;
+
+    struct gif_dest gif_dest;
+
+    write_raster_uncompressed_init(&gif_dest, init_bits, outfile);
+
+    g_outfile = outfile;
+
+    eof = FALSE;
+    while (!eof) {
+        int gifpixel;
+            /* The value for the pixel in the GIF image.  I.e. the colormap
+               index.  Or -1 to indicate "no more pixels."
+            */
+        gifpixel = GIFNextPixel(pixels, 
+                                input_maxval, alpha, alpha_threshold, cmapP);
+        if (gifpixel == EOF) eof = TRUE;
+        if (!eof) {
+            write_raster_uncompressed_pixel(&gif_dest, gifpixel, outfile);
+        }
+    }
+    write_raster_uncompressed_term(&gif_dest, outfile);
+}
+
+
+
 /******************************************************************************
  *
  * GIF Specific routines
  *
- ******************************************************************************/
-
-/*
- * Number of characters so far in this 'packet'
- */
-static int a_count;
-
-/*
- * Set up the 'byte output' routine
- */
-static void
-char_init()
-{
-        a_count = 0;
-}
-
-/*
- * Define the storage for the packet accumulator
- */
-static char accum[ 256 ];
-
-/*
- * Add a character to the end of the current packet, and if it is 254
- * characters, flush the packet to disk.
- */
-static void
-char_out( c )
-int c;
-{
-        accum[ a_count++ ] = c;
-        if( a_count >= 254 )
-                flush_char();
-}
-
-/*
- * Flush the packet to disk, and reset the accumulator
- */
-static void
-flush_char()
-{
-        if( a_count > 0 ) {
-                fputc( a_count, g_outfile );
-                fwrite( accum, 1, a_count, g_outfile );
-                a_count = 0;
-        }
-}
-
-
+ *****************************************************************************/
 
 static void
-GIFEncode(FILE * const fp, 
-          pixel ** const pixels, pixval const input_maxval,
-          int const GWidth, int const GHeight, 
-          gray ** const alpha, gray const alpha_maxval,
-          int const GInterlace, int const Background, 
-          int const BitsPerPixel, struct cmap * const cmapP,
-          const char comment[]) {
+writeGifHeader(FILE * const fp,
+               int const Width, int const Height, 
+               int const GInterlace, int const Background, 
+               int const BitsPerPixel, struct cmap * const cmapP,
+               const char comment[]) {
 
     int B;
-    int RWidth, RHeight;
-    int LeftOfs, TopOfs;
-    int Resolution;
-    int ColorMapSize;
-    int InitCodeSize;
-    int i;
-
-    Interlace = GInterlace;
-
-    ColorMapSize = 1 << BitsPerPixel;
-
-    RWidth = Width = GWidth;
-    RHeight = Height = GHeight;
-    LeftOfs = TopOfs = 0;
-
-    Resolution = BitsPerPixel;
+    int const Resolution = BitsPerPixel;
+    int const ColorMapSize = 1 << BitsPerPixel;
 
     /* Calculate number of bits we are expecting */
     CountDown = (long)Width * (long)Height;
@@ -782,12 +981,6 @@ GIFEncode(FILE * const fp,
     /* Indicate which pass we are on (if interlace) */
     Pass = 0;
 
-    /* The initial code size */
-    if( BitsPerPixel <= 1 )
-        InitCodeSize = 2;
-    else
-        InitCodeSize = BitsPerPixel;
-
     /* Set up the current x and y position */
     curx = cury = 0;
 
@@ -798,14 +991,14 @@ GIFEncode(FILE * const fp,
         fwrite("GIF87a", 1, 6, fp);
 
     /* Write out the screen width and height */
-    Putword( RWidth, fp );
-    Putword( RHeight, fp );
+    Putword( Width, fp );
+    Putword( Height, fp );
 
     /* Indicate that there is a global colour map */
     B = 0x80;       /* Yes, there is a color map */
 
     /* OR in the resolution */
-    B |= (Resolution - 1) << 5;
+    B |= (Resolution - 1) << 4;
 
     /* OR in the Bits per Pixel */
     B |= (BitsPerPixel - 1);
@@ -819,18 +1012,40 @@ GIFEncode(FILE * const fp,
     /* Byte of 0's (future expansion) */
     fputc( 0, fp );
 
-    /* Write out the Global Color Map */
-    for( i=0; i < ColorMapSize; ++i ) {
-        fputc( cmapP->red[i], fp );
-        fputc( cmapP->green[i], fp );
-        fputc( cmapP->blue[i], fp );
+    {
+        /* Write out the Global Color Map */
+        int i;
+        for( i=0; i < ColorMapSize; ++i ) {
+            fputc( cmapP->red[i], fp );
+            fputc( cmapP->green[i], fp );
+            fputc( cmapP->blue[i], fp );
+        }
     }
-
+        
     if ( cmapP->transparent >= 0 ) 
         write_transparent_color_index_extension(fp, cmapP->transparent);
 
     if ( comment )
         write_comment_extension(fp, comment);
+}
+
+
+
+static void
+GIFEncode(FILE * const fp, 
+          pixel ** const pixels, pixval const input_maxval,
+          int const GWidth, int const GHeight, 
+          gray ** const alpha, gray const alpha_maxval,
+          int const GInterlace, int const Background, 
+          int const BitsPerPixel, struct cmap * const cmapP,
+          const char comment[], bool const nolzw) {
+
+    int const LeftOfs = 0;
+    int const TopOfs = 0;
+    int InitCodeSize;
+    
+    writeGifHeader(fp, GWidth, GHeight, GInterlace, Background,
+                   BitsPerPixel, cmapP, comment);
 
     /* Write an Image separator */
     fputc( ',', fp );
@@ -839,21 +1054,38 @@ GIFEncode(FILE * const fp,
 
     Putword( LeftOfs, fp );
     Putword( TopOfs, fp );
-    Putword( Width, fp );
-    Putword( Height, fp );
+    Putword( GWidth, fp );
+    Putword( GHeight, fp );
 
     /* Write out whether or not the image is interlaced */
-    if( Interlace )
+    if( GInterlace )
         fputc( 0x40, fp );
     else
         fputc( 0x00, fp );
 
+    /* The initial code size */
+    if( BitsPerPixel <= 1 )
+        InitCodeSize = 2;
+    else
+        InitCodeSize = BitsPerPixel;
+
     /* Write out the initial code size */
     fputc( InitCodeSize, fp );
 
-    /* Go and actually compress the data */
-    compress(pixels, input_maxval, alpha, alpha_maxval, cmapP, 
-             InitCodeSize+1, fp);
+    /* Set some global variables for BumpPixel() */
+    Interlace = GInterlace;
+    Width = GWidth;
+    Height = GHeight;
+
+    /* Write the actual raster */
+    if (nolzw)
+        write_raster_uncompressed(pixels, 
+                                  input_maxval, alpha, alpha_maxval, cmapP, 
+                                  InitCodeSize+1, fp);
+    else
+        write_raster_LZW(pixels, 
+                         input_maxval, alpha, alpha_maxval, cmapP, 
+                         InitCodeSize+1, fp);
 
     /* Write out a Zero-length packet (to end the series) */
     fputc( 0, fp );
@@ -865,17 +1097,6 @@ GIFEncode(FILE * const fp,
     fclose( fp );
 }
 
-/*
- * Write out a word to the GIF file
- */
-static void
-Putword( w, fp )
-int w;
-FILE* fp;
-{
-        fputc( w & 0xff, fp );
-        fputc( (w / 256) & 0xff, fp );
-}
 
 
 static int
@@ -1184,7 +1405,8 @@ main(int argc, char *argv[]) {
     /* All set, let's do it. */
     GIFEncode(stdout, pixels, input_maxval, cols, rows, 
               alpha, alpha_maxval, 
-              cmdline.interlace, 0, BitsPerPixel, &cmap, cmdline.comment);
+              cmdline.interlace, 0, BitsPerPixel, &cmap, cmdline.comment,
+              cmdline.nolzw);
 
     ppm_freearray(pixels, rows);
     if (alpha)
