@@ -23,7 +23,6 @@
 #include <string.h>
 #include "ppm.h"
 #include "ppmcmap.h"
-#include "pbmplus.h"
 
 #include "mitsu.h"
 
@@ -540,7 +539,7 @@ mymalloc(bytes)
 
     mem = malloc(bytes);
     if( mem == NULL )
-        pm_error("out of memory allocating %d bytes", bytes);
+        pm_error("out of memory allocating %ld bytes", bytes);
 
     return mem;
 }
